<?php
/**
 * @brief		Musicbox Application Class
 * @author		<a href='http://ipsviet.com'>onlyME</a>
 * @copyright	(c) 2015 onlyME
 * @package		IPS Social Suite
 * @subpackage	Musicbox
 * @since		21 Dec 2015
 * @version		
 */
 
namespace IPS\musicbox;

/**
 * Musicbox Application Class
 */
class _Application extends \IPS\Application
{
	/**
	 * Init
	 *
	 * @return	void
	 */
	public function init()
	{
		\IPS\Output::i()->globalControllers[]  = 'musicbox.AjaxPageLoad';
		
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->jsVars['musicBoxMainHome'] = 1;
		}	
		
		\IPS\Output::i()->jsVars['board_name'] = \IPS\Settings::i()->board_name;

		if ( \IPS\Settings::i()->mbconf_rss )
		{
			if ( !( \IPS\Request::i()->controller == "artists" && \IPS\Request::i()->name ) && \IPS\Request::i()->controller != "profile" )
			{
				$rssTitle = \IPS\Member::loggedIn()->language()->addToStack( 'mb_podcast_genres' );
				$rssUrl = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=index&do=podcast", 'front', 'musicbox_podcast' );		
				\IPS\Output::i()->rssFeeds[$rssTitle] = $rssUrl;				
			}		
		}
	}
	
	/**
	 * [Node] Get Icon for tree
	 *
	 * @note	Return the class for the icon (e.g. 'globe')
	 * @return	string|null
	 */
	protected function get__icon()
	{
		return 'music';
	}	
	
	/**
	 * Default front navigation
	 *
	 * @code
	 	
	 	// Each item...
	 	array(
			'key'		=> 'Example',		// The extension key
			'app'		=> 'core',			// [Optional] The extension application. If ommitted, uses this application	
			'config'	=> array(...),		// [Optional] The configuration for the menu item
			'title'		=> 'SomeLangKey',	// [Optional] If provided, the value of this language key will be copied to menu_item_X
			'children'	=> array(...),		// [Optional] Array of child menu items for this item. Each has the same format.
		)
	 	
	 	return array(
		 	'rootTabs' 		=> array(), // These go in the top row
		 	'browseTabs'	=> array(),	// These go under the Browse tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'browseTabsEnd'	=> array(),	// These go under the Browse tab after all other items on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Browse tab may not exist)
		 	'activityTabs'	=> array(),	// These go under the Activity tab on a new install or when restoring the default configuraiton; or in the top row if installing the app later (when the Activity tab may not exist)
		)
	 * @endcode
	 * @return array
	 */
	public function defaultFrontNavigation()
	{
		return array(
			'rootTabs'		=> array(
				array(
					'key'		=> 'Musicbox',
					'children'	=> array(
						array( 'key' => 'Songs' ),	
						array( 'key' => 'Playlists' ),
						array( 'key' => 'Artists' ),						
						array( 'key' => 'Favorite' ),					
						array( 'key' => 'Search' ),					
					),
				)
			),
			'browseTabs'	=> array(),
			'browseTabsEnd'	=> array(),
			'activityTabs'	=> array()
		);
	}
	
	/*-------------------------------------------------------------------------*/
	// Sidebar: Top Songs
	/*-------------------------------------------------------------------------*/
	public function topSongs()
	{
		$limit = \IPS\Settings::i()->mbconf_sb_topSongs;
		if ( $limit == 0 )
		{
			return;
		}		
		
		if ( \IPS\Settings::i()->mbconf_sb_topSongs_time == 1 )
		{
			$categories = array();
			$cwhere = array();
			$cwhere[] = array( 'app=? and perm_type=? and (' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\musicbox\Category::$permissionMap['read'], \IPS\Member::loggedIn()->groups ) . ' OR ' . 'perm_' . \IPS\musicbox\Category::$permissionMap['read'] . '=? )', 'musicbox', 'category', '*' );
			
			if ( \IPS\Settings::i()->mbconf_sb_topSongs_genres != '0' )
			{
				$cwhere[] = array( 'perm_type_id IN(' . \IPS\Settings::i()->mbconf_sb_topSongs_genres . ')' ) ;
			}

			foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', $cwhere ) as $category )
			{
				$categories[]	= $category;
			}
			
			if( !\count( $categories ) )
			{
				return '';
			}

			foreach ( array( 'week' => 'P1W', 'month' => 'P1M', 'year' => 'P1Y', 'all' => NULL ) as $time => $interval )
			{
				$where[] = array( "song_open=?", 1 );
				$where[] = array( 'song_cat IN(' . implode( ',', $categories ) . ')' );
				$where[] = array( 'song_isPlaylist<>?', 1 );
				
				if ( $interval )
				{
					$where[] = array( 'song_started>?', \IPS\DateTime::create()->sub( new \DateInterval( $interval ) )->getTimestamp() );
				}
				
				$ids = array();

				foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', $where, 'song_rating_total DESC, song_views DESC', $limit ) as $vd )
				{
					$ids[] = $vd['song_id'];
				}

				if( \count( $ids ) )
				{
					$$time = new \IPS\Patterns\ActiveRecordIterator(
						\IPS\Db::i()->select(
							'*',
							'musicbox_songs',
							'song_id IN(' . implode( ',', $ids ) . ')',
							'song_rating_total DESC, song_views DESC'
						),
						'IPS\musicbox\Song'
					);
				}
				else
				{
					$$time = array();
				}			
			}
		
			return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->TopSongs( $week, $month, $year, $all );
		}
		else
		{
			if ( \IPS\Settings::i()->mbconf_sb_topSongs_genres != '0' )
			{
				$where[] = array( 'song_cat IN(' . \IPS\Settings::i()->mbconf_sb_topSongs_genres . ')' ) ;
			}
			$where[] = array( 'song_isPlaylist<>?', 1 );
			$where[] = array( "song_open=?", 1 );
			$data = \IPS\musicbox\Song::getItemsWithPermission( $where, 'song_views DESC', $limit, 'read', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
			if ( \count($data) > 0 )
			{
				return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->TopSongs( 0, 0, 0, 0, $data);
			}
			return null;
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Sidebar: Top Playlists
	/*-------------------------------------------------------------------------*/
	public function topPlaylists()
	{
		$limit = \IPS\Settings::i()->mbconf_sb_topPlaylists;
		if ( $limit == 0 )
		{
			return;
		}	
		if ( \IPS\Settings::i()->mbconf_sb_topSongs_time == 1 )
		{		
			$categories = array();
			$cwhere = array();
			$cwhere[] = array( 'app=? and perm_type=? and (' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\musicbox\Category::$permissionMap['read'], \IPS\Member::loggedIn()->groups ) . ' OR ' . 'perm_' . \IPS\musicbox\Category::$permissionMap['read'] . '=? )', 'musicbox', 'category', '*' );
			
			if ( \IPS\Settings::i()->mbconf_sb_topPlaylists_genres != '0' )
			{
				$cwhere[] = array( 'perm_type_id IN(' . \IPS\Settings::i()->mbconf_sb_topPlaylists_genres . ')' ) ;
			}
			
			foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', $cwhere ) as $category )
			{
				$categories[]	= $category;
			}

			if( !\count( $categories ) )
			{
				return '';
			}

			foreach ( array( 'week' => 'P1W', 'month' => 'P1M', 'year' => 'P1Y', 'all' => NULL ) as $time => $interval )
			{
				$where[] = array( "song_open=?", 1 );
				$where[] = array( 'song_cat IN(' . implode( ',', $categories ) . ')' );
				$where[] = array( 'song_isPlaylist=?', 1 );

				if ( $interval )
				{
					$where[] = array( 'song_started>?', \IPS\DateTime::create()->sub( new \DateInterval( $interval ) )->getTimestamp() );
				}
				
				$ids = array();

				foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', $where, 'song_views DESC', $limit ) as $vd )
				{
					$ids[] = $vd['song_id'];
				}

				if( \count( $ids ) )
				{
					$$time = new \IPS\Patterns\ActiveRecordIterator(
						\IPS\Db::i()->select(
							'*',
							'musicbox_songs',
							'song_id IN(' . implode( ',', $ids ) . ')',
							'song_views DESC'
						),
						'IPS\musicbox\Song'
					);
				}
				else
				{
					$$time = array();
				}			
			}
		
			return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->TopPlaylists( $week, $month, $year, $all );
		}
		else
		{
			if ( \IPS\Settings::i()->mbconf_sb_topPlaylists_genres != '0' )
			{
				$where[] = array( 'song_cat IN(' . \IPS\Settings::i()->mbconf_sb_topPlaylists_genres . ')' ) ;
			}
			$where[] = array( 'song_isPlaylist=?', 1 );
			$where[] = array( "song_open=?", 1 );
			$data = \IPS\musicbox\Song::getItemsWithPermission( $where, 'song_views DESC', $limit, 'read', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );
			
			if ( \count($data) > 0 )
			{
				return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->TopPlaylists( 0, 0, 0, 0, $data);
			}
			return null;			
		}		
	}
	
	/*-------------------------------------------------------------------------*/
	// Sidebar: Top Artists
	/*-------------------------------------------------------------------------*/
	public function topArtists()
	{
		$limit = \IPS\Settings::i()->mbconf_sb_topArtists;
		if ( $limit == 0 )
		{
			return;
		}			
		$select = \IPS\Db::i()->select( '*', 'musicbox_artists', array(), 'artist_rating_average DESC, artist_rating_total DESC', array( 0, $limit ) );

		if ( $select->count( TRUE ) )
		{
			foreach( $select as $row )
			{
				$artists[] = \IPS\musicbox\Artist::constructFromData($row);
			}			
			return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->TopArtists( $artists );
		}
		
		return;
	}
	
	/*-------------------------------------------------------------------------*/
	// Sidebar: Genres
	/*-------------------------------------------------------------------------*/
	public function genres($genID)
	{
		if ( \count( \IPS\musicbox\Category::roots() ) > 0 )
		{
			return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->Genres($genID);
		}
		return;
	}	
	
	/*-------------------------------------------------------------------------*/
	// Sidebar: Recommended songs
	/*-------------------------------------------------------------------------*/
	public function recommended( $song )
	{
		$limit = \IPS\Settings::i()->mbconf_sb_recommended;
		
		if ( $limit == 0 )
		{
			return;
		}
		
		$data = array();
		
		if ( $song->tags() || $song->prefix( FALSE ) )
		{
			$data = \IPS\Dispatcher::i()->dispatcherController->getSimilarContent( $limit );
		}
		
		if ( \count( $data ) < $limit )
		{
			$where[] = array( "song_open=?", 1 );
			
			$where[] = array( "song_isPlaylist=? AND song_id<>?", $song->isPlaylist, $song->id );
			
			$singers = explode( ",", $song->singer );
			
			if( \count( $singers ) > 0 )
			{
				foreach( $singers as $name )
				{
					$where[] = array( "LOWER(song_singer) LIKE CONCAT('%', ?, '%')", $name );
				}
			}
			
			$songs = \IPS\musicbox\Song::getItemsWithPermission( $where, 'RAND()', $limit );
			
			if ( \count( $songs ) == 0 || \count( $songs ) < $limit )
			{
				$songs = \IPS\musicbox\Song::getItemsWithPermission( array( "song_isPlaylist=? AND song_id<>? AND song_cat=?", $song->isPlaylist, $song->id, $song->cat ), 'RAND()', $limit );
			}
			
			if ( \count( $songs ) > 0 )
			{
				foreach( $songs as $s )
				{
					$data[ $s->id ] = $s;
				}
			}
		}
		
		return \IPS\Theme::i()->getTemplate( 'sidebar', 'musicbox', 'front' )->Recommended( $data );
	}	
	
	/*-------------------------------------------------------------------------*/
	// Get file size
	/*-------------------------------------------------------------------------*/	
	public function remotefileSize( $url ) 
	{	
		$result = -1;

		$curl = curl_init( $url );

		curl_setopt( $curl, CURLOPT_NOBODY, true );
		curl_setopt( $curl, CURLOPT_HEADER, true );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $curl, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, FALSE);
		
		$data = curl_exec( $curl );
			
		curl_close( $curl );

		if( $data ) 
		{
			$result = 0;

			if( preg_match( "/^HTTP\/1\.[01] (\d\d\d)/", $data, $matches ) ) {
				$status = (int)$matches[1];
			}

			if( preg_match( "/Content-Length: (\d+)/", $data, $matches ) ) {
				$content_length = (int) $matches[1];
			}
			if( $status == 200 || ($status > 300 && $status <= 308) ) {
				$result = $content_length;
			}
		}

		return $result;
	}	
	
	
	
	
	
	
	
	
}