<?php

namespace IPS\musicbox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _File
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'musicbox_songs', 'song_external=0 AND song_file IS NOT NULL' )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$song = \IPS\Db::i()->select( '*', 'musicbox_songs', "song_external=0 AND song_file IS NOT NULL", 'song_id', array( $offset, 1 ) )->first();
		
		try
		{		
			$file = \IPS\File::get( $oldConfiguration ?: 'musicbox_File', $song['song_file'] )->move( $storageConfiguration );
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_file' => (string) $file ), array( 'song_id=?', $song['song_id'] ) );	
			
			$demo = \IPS\File::get( $oldConfiguration ?: 'musicbox_File', $song['song_demo'] )->move( $storageConfiguration );
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_demo' => (string) $demo ), array( 'song_id=?', $song['song_id'] ) );				
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}		
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$song = \IPS\Db::i()->select( '*', 'musicbox_songs', "song_external=0 AND song_file IS NOT NULL AND song_file != ''", 'song_id', array( $offset, 1 ) )->first();
		if ( $new = \IPS\File::repairUrl( $song['song_file'] ) )
		{
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_file' => $new ), array( 'song_id=?', $song['song_id'] ) );
		}
		
		if (  $song['song_demo'] and $new = \IPS\File::repairUrl( $song['song_demo'] ) )
		{
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_demo' => $new ), array( 'song_id=?', $song['song_id'] ) );
		}			
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			$record	= \IPS\Db::i()->select( '*', 'musicbox_songs', array( '( song_file=? OR song_demo=? ) AND song_external=?', (string) $file, (string) $file, 0 ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', "song_external=0 AND song_file IS NOT NULL AND song_file != ''" ) as $song )
		{
			try
			{
				\IPS\File::get( 'musicbox_File', $song['song_file'] )->delete();			
				\IPS\File::get( 'musicbox_File', $song['song_demo'] )->delete();			
			}
			catch( \Exception $e ){}
		}
	}
}