<?php

namespace IPS\musicbox\extensions\core\FileStorage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Editor Extension: Icons
 */
class _Playlist
{
	/**
	 * Count stored files
	 *
	 * @return	int
	 */
	public function count()
	{
		return \IPS\Db::i()->select( 'COUNT(*)', 'musicbox_songs', array( 'song_playlist_image<>? AND song_playlist_image IS NOT NULL', '' ) )->first();
	}
	
	/**
	 * Move stored files
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @param	int			$storageConfiguration	New storage configuration ID
	 * @param	int|NULL	$oldConfiguration		Old storage configuration ID
	 * @throws	\Underflowexception				When file record doesn't exist. Indicating there are no more files to move
	 * @return	void
	 */
	public function move( $offset, $storageConfiguration, $oldConfiguration=NULL )
	{
		$Playlist = \IPS\Db::i()->select( '*', 'musicbox_songs', "song_playlist_image IS NOT NULL AND song_playlist_image != ''", 'song_id', array( $offset, 1 ) )->first();
		
		try
		{		
			$thumbnail			= \IPS\File::get( $oldConfiguration ?: 'musicbox_Playlist', $Playlist['song_playlist_image'] )->move( $storageConfiguration );
			$thumbnail_small	= \IPS\File::get( $oldConfiguration ?: 'musicbox_Playlist', $Playlist['song_playlist_image_thumb'] )->move( $storageConfiguration );
			
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_playlist_image' => (string) $thumbnail, 'song_playlist_image_thumb' 	=> (string) $thumbnail_small ), array( 'song_id=?', $Playlist['song_id'] ) );		
		}
		catch( \Exception $e )
		{
			/* Any issues are logged */
		}		
	}

	/**
	 * Fix all URLs
	 *
	 * @param	int			$offset					This will be sent starting with 0, increasing to get all files stored by this extension
	 * @return void
	 */
	public function fixUrls( $offset )
	{
		$Playlist = \IPS\Db::i()->select( '*', 'musicbox_songs', "song_playlist_image IS NOT NULL AND song_playlist_image != ''", 'song_id', array( $offset, 1 ) )->first();
		if ( $new = \IPS\File::repairUrl( $Playlist['song_playlist_image'] ) )
		{
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_playlist_image' => $new ), array( 'song_id=?', $Playlist['song_id'] ) );
		}
		
		if ( $Playlist['song_playlist_image_thumb'] and $new = \IPS\File::repairUrl( $Playlist['song_playlist_image_thumb'] ) )
		{
			\IPS\Db::i()->update( 'musicbox_songs', array( 'song_playlist_image_thumb' => $new ), array( 'song_id=?', $Playlist['song_id'] ) );
		}		
	}
	
	/**
	 * Check if a file is valid
	 *
	 * @param	\IPS\Http\Url	$file		The file to check
	 * @return	bool
	 */
	public function isValidFile( $file )
	{
		try
		{
			\IPS\Db::i()->select( '*', 'musicbox_songs', array( 'song_playlist_image=? OR song_playlist_image_thumb=?', $file, $file ) )->first();
			return TRUE;
		}
		catch ( \UnderflowException $e )
		{
			return FALSE;
		}
	}

	/**
	 * Delete all stored files
	 *
	 * @return	void
	 */
	public function delete()
	{
		foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', "song_playlist_image IS NOT NULL AND song_playlist_image != ''" ) as $Playlist )
		{
			try
			{
				\IPS\File::get( 'musicbox_Playlist', $Playlist['song_playlist_image'] )->delete();
				\IPS\File::get( 'musicbox_Playlist', $Playlist['song_playlist_image_thumb'] )->delete();				
			}
			catch( \Exception $e ){}
		}
	}
}