<?php
/**
 * @brief		ACP Live Search Extension
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Musicbox
 * @since		18 Sep 2019
 */

namespace IPS\musicbox\extensions\core\LiveSearch;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	ACP Live Search Extension
 */
class _Livesearch
{	
	/**
	 * Constructor
	 *
	 * @return	void
	 */
	public function __construct()
	{
		/* Check Permissions */
		return \IPS\Member::loggedIn()->hasAcpRestriction( 'musicbox', 'musicbox', 'categories_manage' );
	}
	
	/**
	 * Get the search results
	 *
	 * @param	string	$searchTerm	Search Term
	 * @return	array 	Array of results
	 */
	public function getResults( $searchTerm )
	{
		if( !$this->hasAccess() )
		{
			return array();
		}

		/* Init */
		$results = array();
		$searchTerm = mb_strtolower( $searchTerm );
		
		/* Search Category */
		$categories = \IPS\Db::i()->select(
						"*",
						'musicbox_categories',
						array( "cclub_id IS NULL AND word_custom LIKE CONCAT( '%', ?, '%' ) AND lang_id=?", $searchTerm, \IPS\Member::loggedIn()->language()->id ),
						NULL,
						NULL
				)->join(
						'core_sys_lang_words',
						"word_key=CONCAT( 'musicbox_category_', cid )"
					);
		
		foreach ( $categories as $category )
		{
			$category = \IPS\musicbox\Category::constructFromData( $category );
			
			$results[] = \IPS\Theme::i()->getTemplate( 'livesearch', 'musicbox', 'admin' )->category( $category );
		}
		
		/* Search Artist */
		$artists = \IPS\Db::i()->select(
					"*",
					'musicbox_artists',
					array( "artist_name LIKE CONCAT( '%', ?, '%' )", $searchTerm ),
					NULL,
					NULL
				);
		
		foreach ( $artists as $artist )
		{
			$artist = \IPS\musicbox\Artist::constructFromData( $artist );
			
			$results[] = \IPS\Theme::i()->getTemplate( 'livesearch', 'musicbox', 'admin' )->artist( $artist );
		}		
		
		return $results;
	}

	/**
	 * Check we have access
	 *
	 * @return	bool
	 */
	public function hasAccess()
	{
		/* Check Permissions */
		return TRUE;
	}
	
	/**
	 * Is default for current page?
	 *
	 * @return	bool
	 */
	public function isDefault()
	{
		return \IPS\Dispatcher::i()->application->directory == 'musicbox';
	}
}