<?php
/**
 * @brief		Song
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Musicbox
 * @since		05 Apr 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\musicbox\extensions\nexus\Item;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Song
 */
class _Song extends \IPS\nexus\Invoice\Item\Purchase // or \IPS\nexus\Invoice\Item\Purchase
{
	/**
	 * @brief	Application
	 */
	public static $application = 'musicbox';
	
	/**
	 * @brief	Application
	 */
	public static $type = 'song';
	
	/**
	 * @brief	Icon
	 */
	public static $icon = 'music';
	
	/**
	 * @brief	Title
	 */
	public static $title = 'song';
	
	/**
	 * Image
	 *
	 * @return |IPS\File|NULL
	 */
	public function image()
	{
		try
		{
			$song = \IPS\musicbox\Song::load( $this->id );
			return $song->thumbnail(1);
		}
		catch ( \Exception $e )
		{
			return NULL;
		}
	}
	
	/**
	 * Image
	 *
	 * @param	\IPS\nexus\Purchase	$purchase	The purchase
	 * @return |IPS\File|NULL
	 */
	public static function purchaseImage( \IPS\nexus\Purchase $purchase )
	{
		try
		{			
			$song = \IPS\musicbox\Song::load( $purchase->item_id );
			return $song->thumbnail(1);		
		}
		catch ( \Exception $e )
		{
			return NULL;
		}
	}	
	
	/**
	 * Get Client Area Page HTML
	 *
	 * @return	array( 'packageInfo' => '...', 'purchaseInfo' => '...' )
	 */
	public static function clientAreaPage( \IPS\nexus\Purchase $purchase )
	{
		try
		{
			$song = \IPS\musicbox\Song::load( $purchase->item_id );
			
			return array( 'packageInfo' => \IPS\Theme::i()->getTemplate( 'nexus', 'musicbox' )->Info( $song ), 'purchaseInfo' => \IPS\Theme::i()->getTemplate( 'nexus', 'musicbox' )->PurchaseInfo( $song ) );
		}
		catch ( \OutOfRangeException $e ) { }
		
		return NULL;
	}
	
	/**
	 * Get ACP Page HTML
	 *
	 * @return	string
	 */
	public static function acpPage( \IPS\nexus\Purchase $purchase )
	{
		try
		{
			$song = \IPS\musicbox\Song::load( $purchase->item_id );
			return \IPS\Theme::i()->getTemplate( 'nexus', 'musicbox' )->fileInfo( $song );
		}
		catch ( \OutOfRangeException $e ) { }
		
		return NULL;
	}
	
	/**
	 * URL
	 *
	 * @return |IPS\Http\Url|NULL
	 */
	public function url()
	{
		try
		{
			return \IPS\musicbox\Song::load( $this->id )->url();
		}
		catch ( \OutOfRangeException $e )
		{
			return NULL;
		}
	}
	
	/**
	 * ACP URL
	 *
	 * @return |IPS\Http\Url|NULL
	 */
	public function acpUrl()
	{
		return $this->url();
	}	
}