<?php
/**
 * @brief		Feeds
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	Forums
 * @since		04 Feb 2014
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\musicbox\modules\admin\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * rss
 */
class _rss extends \IPS\Node\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
		
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'rss_manage' );
		parent::execute();
	}

	/**
	 * Node Class
	 */
	protected $nodeClass = '\IPS\musicbox\Feed';
	
	/**
	 * Get Root Buttons
	 *
	 * @return	array
	 */
	public function _getRootButtons()
	{
		$buttons = parent::_getRootButtons();
		
		if ( isset( $buttons['add'] ) )
		{
			$buttons['add']['link'] = $buttons['add']['link']->setQueryString( '_new', TRUE );
		}
		
		return $buttons;
	}
	
	/**
	 * Add/Edit Form
	 *
	 * @return void
	 */
	protected function form()
	{
		$nodeClass = $this->nodeClass;
		if ( !\IPS\Request::i()->id and $nodeClass::canAddRoot() )
		{
			\IPS\Output::i()->output = new \IPS\Helpers\Wizard( array(
				'mbrss_import_url' => function( $data )
				{
					$form = new \IPS\Helpers\Form;
					$form->add( new \IPS\Helpers\Form\Url( 'mbrss_import_url', NULL, TRUE ) );
					$form->add( new \IPS\Helpers\Form\Text( 'mbrss_import_auth_user' ) );
					$form->add( new \IPS\Helpers\Form\Password( 'mbrss_import_auth_pass' ) );
					if ( $values = $form->values() )
					{
						try
						{
							$request = $values['mbrss_import_url']->request();
							
							if ( $values['mbrss_import_auth_user'] or $values['mbrss_import_auth_pass'] )
							{
								$request = $request->login( $values['mbrss_import_auth_user'], $values['mbrss_import_auth_pass'] );
							}
							
							$response = $request->get();
							
							if ( $response->httpResponseCode == 401 )
							{
								$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'mbrss_import_auth' );
								return $form;
							}
							
							$response = $response->decodeXml();
							if ( !( $response instanceof \IPS\Xml\Rss ) and !( $response instanceof \IPS\Xml\Atom ) )
							{
								$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'mbrss_import_invalid' );
								return $form;
							}
						}
						catch ( \IPS\Http\Request\Exception $e )
						{
							$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'form_url_bad' );
							return $form;
						}
						catch ( \Exception $e )
						{
							$form->error = \IPS\Member::loggedIn()->language()->addToStack( 'mbrss_import_invalid' );
							return $form;
						}
						
						return $values;
					}
					return $form;
				},
				'mbrss_import_preview' => function( $data )
				{
					if ( isset( \IPS\Request::i()->continue ) )
					{
						return $data;
					}
					
					$request = \IPS\Http\Url::external( $data['mbrss_import_url'] )->request();
					if ( $data['mbrss_import_auth_user'] or $data['mbrss_import_auth_pass'] )
					{
						$request = $request->login( $data['mbrss_import_auth_user'], $data['mbrss_import_auth_pass'] );
					}
					return \IPS\Theme::i()->getTemplate( 'feeds' )->importPreview( $request->get()->decodeXml()->articles() );
				},
				'mbrss_import_details' => function ( $data )
				{
					$request = \IPS\Http\Url::external( $data['mbrss_import_url'] )->request();
					if ( $data['mbrss_import_auth_user'] or $data['mbrss_import_auth_pass'] )
					{
						$request = $request->login( $data['mbrss_import_auth_user'], $data['mbrss_import_auth_pass'] );
					}
										
					$form = new \IPS\Helpers\Form;
					$form->add( new \IPS\Helpers\Form\Node( 'mbrss_import_genre_id', NULL, TRUE, array( 'class' => 'IPS\musicbox\Category' ) ) );
					$form->add( new \IPS\Helpers\Form\Member( 'mbrss_import_mid', \IPS\Member::loggedIn(), TRUE ) );
					$form->add( new \IPS\Helpers\Form\Text( 'mbrss_import_artist', NULL, FALSE ) );
					
					if ( $values = $form->values() )
					{
						$import = new \IPS\musicbox\Feed;
						$import->enabled = TRUE;
						$import->title = $request->get()->decodeXml()->title();
						$import->url = (string) $data['mbrss_import_url'];
						$import->genre_id = $values['mbrss_import_genre_id']->_id;
						$import->mid = $values['mbrss_import_mid']->member_id;
						$import->artist = $values['mbrss_import_artist'];
						$import->pergo = 10;
						$import->time = 30;
						$import->auth = ( $data['mbrss_import_auth_user'] or $data['mbrss_import_auth_pass'] );
						$import->auth_user = ( $data['mbrss_import_auth_user'] or $data['mbrss_import_auth_pass'] ) ? $data['mbrss_import_auth_user'] : NULL;
						$import->auth_pass = ( $data['mbrss_import_auth_user'] or $data['mbrss_import_auth_pass'] ) ? $data['mbrss_import_auth_pass'] : NULL;
						$import->save();
						try
						{
							$import->run();
						}
						catch( \Exception $e )
						{
							\IPS\Output::i()->error( 'rss_run_error', '3BIMMB107/3', 500, '' );
						}
						\IPS\Db::i()->update( 'core_tasks', array( 'enabled' => 1 ), array( '`key`=?', 'rssimport' ) );
						\IPS\Session::i()->log( 'acplog__node_created', array( (string) $import->title => TRUE, (string) $import->title => FALSE ) );
						\IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=musicbox&module=musicbox&controller=rss' ) );
					}
					
					return $form;
				}
			), \IPS\Http\Url::internal('app=musicbox&module=musicbox&controller=rss&do=form') );
		}
		else
		{
			return parent::form();
		}
	}
	
	/**
	 * Run
	 *
	 * @return	void
	 */
	public function run()
	{
		\IPS\Session::i()->csrfCheck();
		
		\IPS\Dispatcher::i()->checkAcpPermission( 'rss_run' );
		
		try
		{
			$feed = \IPS\musicbox\Feed::load( \IPS\Request::i()->id );
			$feed->run();
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB107/1', 404, '' );
		}
		catch ( \Exception $e )
		{
			\IPS\Output::i()->error( 'rss_run_error', '3BIMMB107/2', 500, '' );
		}
		
		\IPS\Session::i()->log( 'acplog__mbrss_ran', array( $feed->title => FALSE ) );
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal('app=musicbox&module=musicbox&controller=rss' ) );
	}
}