<?php


namespace IPS\musicbox\modules\admin\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
		
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$form = new \IPS\Helpers\Form;
		
		/* Index */
		$form->addTab('mb_conf_general');
		$form->addHeader('mb_conf_general');
		$form->add( new \IPS\Helpers\Form\Select( 'mbconf_autoplay_default', \IPS\Settings::i()->mbconf_autoplay_default ? \IPS\Settings::i()->mbconf_autoplay_default : 'off', FALSE, array( 'options' => array( 'off' => 'mb_conf_apoff', 'on' => 'mb_conf_apon' ) ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_rss', \IPS\Settings::i()->mbconf_rss, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_update_url', \IPS\Settings::i()->mbconf_update_url, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_global_search', \IPS\Settings::i()->mbconf_global_search, FALSE, array() ) );		
		
		$form->addHeader('mb_conf_bottomplayer');	
		$form->add( new \IPS\Helpers\Form\Select( 'mbconf_playerbar_theme', \IPS\Settings::i()->mbconf_playerbar_theme ? \IPS\Settings::i()->mbconf_playerbar_theme : 'light', FALSE, array( 'options' => array( 'light' => 'mb_conf_bottomplayer_light', 'dark' => 'mb_conf_bottomplayer_dark' ) ) ) );
		$form->add( new \IPS\Helpers\Form\Radio( 'mbconf_playerbar_all', \IPS\Settings::i()->mbconf_playerbar_all, FALSE, array(
			'options' => array(
				0	=>	'mb_playerbar_mb',
				1	=>	'mb_playerbar_all',
				2	=>	'mb_playerbar_song',
			)
		) ) );
		
		$form->addHeader('mb_conf_index');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_index_playlists', \IPS\Settings::i()->mbconf_index_playlists ? \IPS\Settings::i()->mbconf_index_playlists : 12, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_index_songs', \IPS\Settings::i()->mbconf_index_songs ? \IPS\Settings::i()->mbconf_index_songs : 20, TRUE, array() ) );

		$form->addTab('mb_conf_songs');	
		$form->addHeader('mb_conf_upload');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_maxfilesize', \IPS\Settings::i()->mbconf_maxfilesize ? \IPS\Settings::i()->mbconf_maxfilesize : 20, TRUE, array() ) );
		$form->addHeader('mb_conf_download');		
		$form->add( new \IPS\Helpers\Form\Text( 'mbconf_song_name', \IPS\Settings::i()->mbconf_song_name ? \IPS\Settings::i()->mbconf_song_name : "{name}", TRUE, array() ) );		
		$form->addHeader('mb_conf_playlist');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_maxSongs', \IPS\Settings::i()->mbconf_maxSongs ? \IPS\Settings::i()->mbconf_maxSongs : 100, FALSE, array() ) );		

		$form->addTab('mb_conf_sidebar');
		$form->addHeader('mbconf_sb_topPlaylists');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_sb_topPlaylists', \IPS\Settings::i()->mbconf_sb_topPlaylists ? \IPS\Settings::i()->mbconf_sb_topPlaylists : 10, TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Node( 'mbconf_sb_topPlaylists_genres', ( \IPS\Settings::i()->mbconf_sb_topPlaylists_genres and \IPS\Settings::i()->mbconf_sb_topPlaylists_genres != '*' ) ? \IPS\Settings::i()->mbconf_sb_topPlaylists_genres : 0, FALSE, array( 'class' => 'IPS\musicbox\Category', 'multiple' => TRUE, 'zeroVal' => 'all' ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_sb_topPlaylists_time', \IPS\Settings::i()->mbconf_sb_topPlaylists_time ) );

		$form->addHeader('mbconf_sb_topSongs');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_sb_topSongs', \IPS\Settings::i()->mbconf_sb_topSongs ? \IPS\Settings::i()->mbconf_sb_topSongs : 10, TRUE, array() ) );		
		$form->add( new \IPS\Helpers\Form\Node( 'mbconf_sb_topSongs_genres', ( \IPS\Settings::i()->mbconf_sb_topSongs_genres and \IPS\Settings::i()->mbconf_sb_topSongs_genres != '*' ) ? \IPS\Settings::i()->mbconf_sb_topSongs_genres : 0, FALSE, array( 'class' => 'IPS\musicbox\Category', 'multiple' => TRUE, 'zeroVal' => 'all' ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_sb_topSongs_time', \IPS\Settings::i()->mbconf_sb_topSongs_time ) );
		
		$form->addHeader('mbconf_sb_topArtists');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_sb_topArtists', \IPS\Settings::i()->mbconf_sb_topArtists ? \IPS\Settings::i()->mbconf_sb_topArtists : 9, TRUE, array() ) );		
		
		$form->addHeader('mbconf_sb_recommended');	
		$form->add( new \IPS\Helpers\Form\Number( 'mbconf_sb_recommended', \IPS\Settings::i()->mbconf_sb_recommended ? \IPS\Settings::i()->mbconf_sb_recommended : 10, TRUE, array() ) );		
				
		if ( \IPS\Application::appIsEnabled( 'nexus' ) )
		{
			$form->addTab( 'paid_song_settings' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'mbconf_nexus_on', \IPS\Settings::i()->mbconf_nexus_on, FALSE, array( 'togglesOn' => array( 'mbconf_nexus_tax', 'mbconf_nexus_percent', 'mbconf_nexus_transfee', 'mbconf_nexus_gateways', 'mbconf_nexus_can_add' ) ) ) );		
			$form->add( new \IPS\Helpers\Form\Select( 'mbconf_nexus_can_add', \IPS\Settings::i()->mbconf_nexus_can_add ? array_filter( explode( ',', \IPS\Settings::i()->mbconf_nexus_can_add ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ), NULL, NULL, NULL, 'mbconf_nexus_can_add' ) );			
			$form->add( new \IPS\Helpers\Form\Node( 'mbconf_nexus_tax', \IPS\Settings::i()->mbconf_nexus_tax ?:0, FALSE, array( 'class' => '\IPS\nexus\Tax', 'zeroVal' => 'do_not_tax' ), NULL, NULL, NULL, 'mbconf_nexus_tax' ) );
			$form->add( new \IPS\Helpers\Form\Number( 'mbconf_nexus_percent', \IPS\Settings::i()->mbconf_nexus_percent, FALSE, array( 'min' => 0, 'max' => 100 ), NULL, NULL, '%', 'mbconf_nexus_percent' ) );
			$form->add( new \IPS\nexus\Form\Money( 'mbconf_nexus_transfee', json_decode( \IPS\Settings::i()->mbconf_nexus_transfee, TRUE ), FALSE, array(), NULL, NULL, NULL, 'mbconf_nexus_transfee' ) );
			$form->add( new \IPS\Helpers\Form\Node( 'mbconf_nexus_gateways', ( \IPS\Settings::i()->mbconf_nexus_gateways ) ? explode( ',', \IPS\Settings::i()->mbconf_nexus_gateways ) : 0, FALSE, array( 'class' => '\IPS\nexus\Gateway', 'zeroVal' => 'no_restriction', 'multiple' => TRUE ), NULL, NULL, NULL, 'mbconf_nexus_gateways' ) );
		}
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			$form->saveAsSettings( $values );
		}
		
		/* Output */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('settings');
		\IPS\Output::i()->output = $form;		
	}
	
	// Create new methods with the same name as the 'do' parameter which should execute it
}