<?php


namespace IPS\musicbox\modules\admin\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * tools
 */
class _tools extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
		
		
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'tools_manage' );
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$form = new \IPS\Helpers\Form;
		
		$allArtists = array();
		try
		{
			foreach( \IPS\musicbox\Artist::roots() as $artist )
			{
				$allArtists[] = $artist->name;
			}
		}
		catch( \OutOfRangeException $e ) {}
		
		$form->add( new \IPS\Helpers\Form\Text( 'mb_massupload_artists', NULL, FALSE, array( 'autocomplete' => array( 'source' => $allArtists, 'unique' => true, 'freeChoice' => true, 'minimized' => false ) ) ) );
	
		$form->add( new \IPS\Helpers\Form\Select( 'mb_massupload_to', 'cat', FALSE, array(
			'options' 	=> array( 'cat' => 'mb_massupload_to_cat', 'pl' => 'mb_massupload_to_pl' ),
			'toggles' 	=> array( 'cat' => array('mb_massupload_genres'), 'pl' => array('mb_massupload_playlist') )
		) ) );			
		$form->add( new \IPS\Helpers\Form\Node( 'mb_massupload_genres', NULL, FALSE, array( 'class' => 'IPS\musicbox\Category', 'multiple' => FALSE, 'clubs' => TRUE ), NULL, NULL, NULL, 'mb_massupload_genres' ) );
		$form->add( new \IPS\Helpers\Form\Number( 'mb_massupload_playlist', NULL, FALSE, array(), NULL, NULL, NULL, 'mb_massupload_playlist' ) );
		
		$options['upload'] = 'musicbox_file_upload';
		$options['external'] = 'musicbox_file_external';
		$toggles = array( 'upload' => array( 'mb_upload' ), 'external' => array( 'mb_external' ) );		
		$form->add( new \IPS\Helpers\Form\Select( 'musicbox_file_from', 'upload', FALSE, array( 'options' => $options, 'toggles' => $toggles ) ) );
		
		$form->add( new \IPS\Helpers\Form\Upload( 'mb_massupload_files', NULL, TRUE, array( 
			'storageExtension' 	=> 'musicbox_File', 
			'allowedFileTypes'	=> array( 'mp3', 'wma', 'm4a' ),
			'multiple'			=> TRUE,
			'maxFileSize'		=> \IPS\Settings::i()->mbconf_maxfilesize ? \IPS\Settings::i()->mbconf_maxfilesize : 20, 
		), NULL, NULL, NULL, 'mb_upload' ) );
		
		$form->add( new \IPS\Helpers\Form\TextArea( 'musicbox_external_urls', NULL, TRUE, array(), NULL, NULL, NULL, 'mb_external' ) );
		
		if ( $values = $form->values() )
		{
			if ( $values['musicbox_file_from'] == 'upload' )
			{
				$files = $values['mb_massupload_files'];
			}
			else
			{
				$files = preg_split('/\r\n|\r|\n/', $values['musicbox_external_urls']);
				$files = array_filter($files);				
			}
				
			foreach( $files as $file )
			{
				$filename = pathinfo( \is_object( $file ) ? $file->originalFilename : $file, PATHINFO_FILENAME );
				
				if ( \intval($values['mb_massupload_playlist']) > 0 )
				{
					try
					{
						$pl = \IPS\musicbox\Song::load( \intval($values['mb_massupload_playlist']) );
						
						if ($pl->isPlaylist != 1)
						{
							\IPS\Output::i()->error( 'mb_error_import_notpl', '2BIMMB106/1', 404, '' );
						}
					}
					catch ( \OutOfRangeException $e )
					{
						\IPS\Output::i()->error( 'node_error', '2BIMMB106/2', 404, '' );
					}
					$song = \IPS\musicbox\Song::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::ts( time() ), $pl->container() );
					$song->inPlaylist = $pl->id;
				}
				else
				{
					if ( ! $values['mb_massupload_genres'] )
					{
						\IPS\Output::i()->error( 'mb_error_import_nogenre', '2BIMMB106/3', 404, '' );
					}
					$song = \IPS\musicbox\Song::createItem( \IPS\Member::loggedIn(), NULL, \IPS\DateTime::ts( time() ), $values['mb_massupload_genres'] );
				}				
				$song->title = $filename;
				$song->title_furl = \IPS\Http\Url::seoTitle( $filename );
				$song->singer = \count( $values['mb_massupload_artists'] ) > 0 ? implode( ", ", $values['mb_massupload_artists'] ) : NULL;
				if ( \is_object( $file ) )
				{
					$song->external = 0;
					$song->file = (string) $file;
					$song->filesize = $file->filesize();
				}
				else
				{
					$song->external = 1;
					$song->file = $file;
					$song->filesize = \IPS\Application::load('musicbox')->remotefileSize( $file );
				}
				
				$song->save();
			}
			
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=tools' ), 'completed' );
		}
		
		/* Output */
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_tools');
		\IPS\Output::i()->output = $form;
	}
	
	public function searchPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		$results = array();
		
		$input = mb_strtolower( \IPS\Request::i()->input );
		
		$where = array( "song_isPlaylist=1 and song_title LIKE CONCAT('%', ?, '%')", $input );
		
		foreach ( \IPS\Db::i()->select( '*', 'musicbox_songs', $where, 'LENGTH(song_title) ASC', array( 0, 20 ) ) as $row )
		{
			$song = \IPS\musicbox\Song::constructFromData( $row );
			
			$results[] = array(
				'id'	=> 	$song->id,
				'value' => 	$song->id,
				'name'	=> 	$song->title . " (ID: {$song->id})",
				'extra'	=> 	$song->singers(),
				'photo'	=> 	$song->thumbnail(),
			);
		}
				
		\IPS\Output::i()->json( $results );		
	}
}