<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * artists
 */
class _artists extends \IPS\Dispatcher\Controller
{
	protected function manage()
	{	
		if ( ! \IPS\Request::i()->name && ! \IPS\Request::i()->id )
		{
			$this->_index();
		}
		else
		{
			$this->_search();
		}
	}
	
	protected function _index()
	{
		$where = array();
		$perpage = 32;

		$page = isset( \IPS\Request::i()->page ) ? \intval( \IPS\Request::i()->page ) : 1;

		if( $page < 1 )
		{
			$page = 1;
		}
		
		$select = \IPS\Db::i()->select( '*', 'musicbox_artists', $where, 'artist_name ASC', array( ( $page - 1 ) * $perpage, $perpage ) );
		$total = (int) \IPS\Db::i()->select( 'COUNT(*)', 'musicbox_artists', $where )->first();
		$pagination = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->pagination(
			\IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ),
			ceil( $total / $perpage ),
			$page,
			$perpage,
			FALSE
		);
		
		foreach( $select as $row )
		{
			$artists[] = \IPS\musicbox\Artist::constructFromData($row);
		}
		
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack( 'artist_list' );
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}			
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ), \IPS\Member::loggedIn()->language()->addToStack( 'artist_list' ) );
		$output = \IPS\Theme::i()->getTemplate( 'artists' )->artists( $artists, $pagination );
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}
	
	protected function _search()
	{
		/* Search by name */
		if ( !isset( \IPS\Request::i()->name ) )
		{
			return;
		}
		
		$name = mb_strtolower( \IPS\Request::i()->name );

		try
		{
			$artist = \IPS\musicbox\Artist::constructFromData( \IPS\Db::i()->select( '*', 'musicbox_artists', array( "LOWER(artist_name)=?", $name ), 'artist_name DESC', 1 )->first() );	
		}
		catch ( \UnderflowException $e )
		{
			$songs = \IPS\musicbox\Song::getItemsWithPermission( array( "LOWER(song_singer) LIKE CONCAT('%', ?, '%')", $name ), 'song_id', 1 );
			
			if ( \count( $songs ) == 1 )
			{
				$data = array( 	'artist_name' 		=> $name,
								'artist_name_furl'	=> \IPS\Http\Url::seoTitle( $name )
				);
				
				$id = \IPS\Db::i()->insert( 'musicbox_artists', $data );
				
				$artist = \IPS\musicbox\Artist::constructFromData( \IPS\Db::i()->select( '*', 'musicbox_artists', array( "artist_id=?", $id ), 'artist_id DESC', 1 )->first() );	
			}
		}
		if ( $artist )
		{
			$profile = new \IPS\musicbox\modules\front\musicbox\profile;
			$profile->info( $artist );
		}
	}
	
	public function add()
	{
		if ( ! \IPS\Member::loggedIn()->inGroup( explode(",", \IPS\Settings::i()->mbconf_artists_perm ) ) )
		{
			\IPS\Output::i()->error( 'musicbox_error_artist_noperm', '2BIMMB107/1', 404, '' );
		}

		$form = new \IPS\Helpers\Form();
		$form->class = 'ipsPad';
		$form->add( new \IPS\Helpers\Form\Text( 'artist_name', "", TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Upload( 'artist_avatar', NULL, FALSE, array( 
			'storageExtension' 	=> 'musicbox_Artist', 
			'image' 			=> TRUE,
			'maxFileSize'		=> 1, 
		) ) );

		$form->add( new \IPS\Helpers\Form\Editor( 'artist_profile', $this ? $this->profile : NULL, FALSE, array( 
			'app' 			=> 'musicbox', 
			'key' 			=> 'Artist', 
			'autoSaveKey' 	=> 'musicbox-new-artist', 
			'attachIds' 	=> NULL
		) ) );
		
		if ( $values = $form->values() )
		{
			$artist	= new \IPS\musicbox\Artist;
			$artist->saveForm( $artist->formatFormValues( $values ) );
			\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ) );
		}		
		
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_newartist' );
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ), \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_newartist' ) );
		\IPS\Output::i()->output= $form;
	}
}