<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * favorite
 */
class _favorite extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		if ( ! \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'musicbox_error_guest', '2BIMMB106/1', 404 );
		}
			
		if ( ! \IPS\Request::i()->mem )
		{
			$this->favoriteURL = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=favorite", 'front', 'musicbox_favorite' );
		}
		else
		{
			$this->favoriteURL = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=favorite&mem=" . \IPS\Request::i()->mem , 'front', 'musicbox_favorite' );
		}
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$page = isset( \IPS\Request::i()->page ) ? \intval( \IPS\Request::i()->page ) : 1;
		$perpage = 30;
		$owner = \IPS\Request::i()->mem ? \IPS\Request::i()->mem : \IPS\Member::loggedIn()->member_id;
		try
		{
			$member = \IPS\Member::load( $owner );
			if ( !$member->member_id )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMMB106/2', 404 );
			}
			$ownerName = $member->name;
		}
		catch ( \OutOfRangeException $e ){}
		
		if ( \IPS\Request::i()->area == 'public' )
		{
			$where = array( 'mpl_mem=? AND mpl_playlistID>?', $owner, 0 );
			$select = \IPS\Db::i()->select( '*', 'musicbox_favorite', $where, 'mpl_id DESC', array( ( $page - 1 ) * $perpage, $perpage ) );
			$total = (int) \IPS\Db::i()->select( 'COUNT(*)', 'musicbox_favorite', $where )->first();
			foreach( $select as $row )
			{
				try
				{
					$playlist = \IPS\musicbox\Song::load( $row['mpl_playlistID'] );
					$data[] = $playlist;
				}
				catch ( \OutOfRangeException $e ){
					\IPS\Db::i()->delete( 'musicbox_favorite', array( 'mpl_mem=? and mpl_playlistID=?', \IPS\Member::loggedIn()->member_id, $row['mpl_playlistID'] ) );
					$broken++;
				}
			}
			if( $broken > 0 )
			{
				\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
			}
		}
		else
		{
			$where = array( 'mpl_mem=? AND mpl_playlistID=?', $owner, 0 );
			$select = \IPS\Db::i()->select( '*', 'musicbox_favorite', $where, 'mpl_position ASC, mpl_name ASC', array( ( $page - 1 ) * $perpage, $perpage ) );
			$total = (int) \IPS\Db::i()->select( 'COUNT(*)', 'musicbox_favorite', $where )->first();

			foreach( $select as $row )
			{
				$row['total'] = $row['mpl_songs'] ? \count(explode(',', $row['mpl_songs'])) : 0;
				$data[] = $row;
			}			
		}
		
		/* Pagination */
		$pagination = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->pagination(
			$this->favoriteURL->setQueryString( array( 'area' => 'public' ) ),
			ceil( $total / $perpage ),
			$page,
			$perpage,
			FALSE
		);
						
		/* Content */
		$content = \IPS\Theme::i()->getTemplate( 'favorite' )->content( $data, $pagination, $this->favoriteURL );
		
		/* Display */
		\IPS\Output::i()->jsFiles	= array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'jquery/jquery-ui.js', 'core', 'interface' ) );	
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_myFav', FALSE, array( 'sprintf' => array( $ownerName ) ) );
		
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}		
		
		\IPS\Output::i()->breadcrumb[] = array( $this->favoriteURL, $title );			
		\IPS\Output::i()->title = $title;
		$output = \IPS\Theme::i()->getTemplate( 'favorite' )->main($content, $this->favoriteURL, $title);
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}
	
	/*-------------------------------------------------------------------------*/
	// Favorite popup
	/*-------------------------------------------------------------------------*/
	public function addFavorite()
	{
		\IPS\Session::i()->csrfCheck();
		
		$where = array( 'mpl_mem=? AND mpl_playlistID=?', \IPS\Member::loggedIn()->member_id, 0 );
		foreach( \IPS\Db::i()->select( '*', 'musicbox_favorite', $where, 'mpl_position ASC, mpl_id DESC') as $row )
		{
			$row['total'] = $row['mpl_songs'] ? \count(explode(',', $row['mpl_songs'])) : 0;
			$playlists[] = $row;
		}

		try
		{
			$song = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e ){}
		
		# Output
		$output = \IPS\Theme::i()->getTemplate( 'favorite' )->add( $song, $playlists );
		
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}
	
	/*-------------------------------------------------------------------------*/
	// Create new playlist
	/*-------------------------------------------------------------------------*/
	protected function createMyPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		try
		{
			$song = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'node_error' ) ) );
			exit;			
		}
				
		if ( $song->isPlaylist == 1 )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_dontaddpl' ) ) );
			exit;
		}
		
		if ( ! \IPS\Request::i()->plname )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_noplname' ) ) );
			exit;
		}
		
		if ( mb_strlen( \IPS\Request::i()->plname ) > 100 )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_longname' ) ) );
			exit;
		}		
		
		$plname = strip_tags( \IPS\Request::i()->plname );
		
		try
		{
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_name=? and mpl_mem=?', $plname, \IPS\Member::loggedIn()->member_id ) )->first();
		}
		catch ( \UnderflowException $e ){}
				
		if ( $myplaylist['mpl_id'] > 0 )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_plexist' ) ) );
			exit;
		}		
		
		$data = array( 	'mpl_name' 			=> $plname,
						'mpl_mem'	 		=> \IPS\Member::loggedIn()->member_id,			
						'mpl_songs'	 		=> $song->id,
		);
		
		\IPS\Db::i()->insert( 'musicbox_favorite', $data );

		$data['total'] = 1;
		$html = \IPS\Theme::i()->getTemplate( 'favorite' )->addRow( $song, $data );
		\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_createdpl' ), 'html' => $html ) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Add a song to my playlist
	/*-------------------------------------------------------------------------*/	
	protected function addToPL()
	{
		\IPS\Session::i()->csrfCheck();
		
		try
		{
			$song = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'node_error' ) ) );
			exit;			
		}
				
		if ( $song->isPlaylist == 1 )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_dontaddpl' ) ) );
			exit;
		}
		
		$playlistID = \IPS\Request::i()->playlistID ? (int) \IPS\Request::i()->playlistID : 0;
		try
		{
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_id=? and mpl_mem=?', $playlistID, \IPS\Member::loggedIn()->member_id ) )->first();
		}
		catch ( \UnderflowException $e ){}
				
		if ( !$myplaylist['mpl_id'] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_nomyplaylist' ) ) );
			exit;
		}		
		
		$allSongs = array();
		$Remove = 0;
		if( $myplaylist['mpl_songs'] )
		{
			if ( !\in_array( $song->id, explode(",",$myplaylist['mpl_songs']) ) )
			{
				$allSongs = explode(",", $myplaylist['mpl_songs'] . "," . $song->id);
				if ( \count($allSongs) > \IPS\Settings::i()->mbconf_maxSongs - 1 )
				{
					\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_maxsongs' ) ) );
					exit;
				}
			}
			else
			{
				$pl = explode(",", $myplaylist['mpl_songs']);
				$allSongs = array_diff($pl, array($song->id));
				$Remove = 1;
			}
		}
		else
		{
			$allSongs = array( $song->id );
		}
		
		\IPS\Db::i()->update( 'musicbox_favorite', array('mpl_songs' => implode(",", $allSongs)), array( 'mpl_id=?', $myplaylist['mpl_id'] ) );
		$msg = $Remove == 0 ? \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_addedtopl' ) : \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_removedfrompl' );
		$button = $Remove == 0 ? \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_removefromthispl' ) : \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_addtothispl' );
		
		\IPS\Output::i()->json( array( 'message' => $msg, 'count' => \count($allSongs), '_button' => $button ) );
	}

	/*-------------------------------------------------------------------------*/
	// Add a playlist to my favorite
	/*-------------------------------------------------------------------------*/		
	protected function addPlayListToFavorite()
	{
		\IPS\Session::i()->csrfCheck();
		
		try
		{
			$song = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'node_error' ) ) );
			exit;			
		}
		
		if ( $song->isPlaylist != 1 )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_dontaddsong' ) ) );
			exit;
		}

		$inPlaylist = $song->playlistInFavorite();
		if ( !$inPlaylist )
		{
			$msg = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_addedplfav' );
			$button = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_remove_pl_fav' );
			\IPS\Db::i()->insert( 'musicbox_favorite', array('mpl_mem' => \IPS\Member::loggedIn()->member_id, 'mpl_playlistID' => $song->id ) );
		}
		else
		{
			$msg = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_removedplfav' );
			$button = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_add_pl_fav' );
			\IPS\Db::i()->delete( 'musicbox_favorite', array( 'mpl_mem=? and mpl_playlistID=?', \IPS\Member::loggedIn()->member_id, $song->id ) );
		}

		\IPS\Output::i()->json( array( 'message' => $msg, '_button' => $button ) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Manage My Playlist
	/*-------------------------------------------------------------------------*/	
	protected function editMyPlaylist()
	{
		$playlistID = \IPS\Request::i()->id ? (int) \IPS\Request::i()->id : 0;
		try
		{
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_id=? and mpl_mem=?', $playlistID, \IPS\Member::loggedIn()->member_id ) )->first();
		}
		catch ( \UnderflowException $e ){}
				
		if ( !$myplaylist['mpl_id'] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_nomyplaylist' ) ) );
			exit;
		}
		
		/* Build form */
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Text( 'mpl_name', $myplaylist['mpl_name'], TRUE ) );
		$form->class = 'ipsPad';

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			\IPS\Db::i()->update( 'musicbox_favorite', array( 'mpl_name' => $values['mpl_name'] ), array( 'mpl_id=?', $myplaylist['mpl_id'] ) );
			\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
		}
		
		# Output
		\IPS\Output::i()->output = (string) $form;
	}
	
	protected function moveMyPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		$position = 0;
		
		if ( \IPS\Request::i()->isAjax() and \IPS\Request::i()->ajax_order )
		{
			$i = 1;
			foreach ( \IPS\Request::i()->ajax_order as $id )
			{
				\IPS\Db::i()->update( 'musicbox_favorite', array( 'mpl_position' => $i ), array( 'mpl_mem=? and mpl_id=?', \IPS\Member::loggedIn()->member_id, $id ) );
				$i++;
			}
			\IPS\Output::i()->json( 'OK' );
			return;
		}
		
		\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
	}
	
	protected function deleteMyPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		$playlistID = \IPS\Request::i()->playlistID ? (int) \IPS\Request::i()->playlistID : 0;
		
		try
		{
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_id=? and mpl_mem=?', $playlistID, \IPS\Member::loggedIn()->member_id ) )->first();
		}
		catch ( \UnderflowException $e ){}
				
		if ( !$myplaylist['mpl_id'] )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_error_nomyplaylist' ) ) );
			exit;
		}		
		
		\IPS\Db::i()->delete( 'musicbox_favorite', array( 'mpl_id=?', $myplaylist['mpl_id'] ) );
		$msg = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_removedplfav' );
		\IPS\Output::i()->json( array( 'message' => $msg ) );
	}	
	
	/*-------------------------------------------------------------------------*/
	// Play
	/*-------------------------------------------------------------------------*/
	protected function play()
	{
		/* My playlist */
		$owner = \IPS\Request::i()->mem ? \IPS\Request::i()->mem : \IPS\Member::loggedIn()->member_id;
		$playlistURL = $this->favoriteURL->setQueryString( array( 'do' => 'play', 'id' => \intval( \IPS\Request::i()->id ) ) );
		
		$songs = \IPS\musicbox\Song::myFavoriteSongs( \IPS\Request::i()->id, $owner );

		$content = \IPS\Theme::i()->getTemplate( 'embed' )->playList( $songs );
		
		/* Display */	
		try
		{
			$member = \IPS\Member::load( $owner );
			if ( !$member->member_id )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMMB106/4', 404 );
			}
			$ownerName = $member->name;
		}
		catch ( \OutOfRangeException $e ){}		
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_myFav', FALSE, array( 'sprintf' => array( $ownerName ) ) );
		
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}		
		\IPS\Output::i()->breadcrumb[] = array( $this->favoriteURL, $title );	
		\IPS\Output::i()->breadcrumb[] = array( $playlistURL, $myplaylist['mpl_name'] );		
		\IPS\Output::i()->title = $title;
		$output = \IPS\Theme::i()->getTemplate( 'favorite' )->main($content, $this->favoriteURL, $title, \intval( \IPS\Request::i()->id ) );	
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}	
}