<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{	
		/* New playlists */
		$newPlaylists = array();
		$newPlaylists = \IPS\musicbox\Song::getItemsWithPermission( array( array( 'song_isPlaylist=?', 1 ) ), 'song_id DESC', \IPS\Settings::i()->mbconf_index_playlists ? \IPS\Settings::i()->mbconf_index_playlists : 18 );
		
		/* New Songs */
		$newSongs = array();
		$newSongs = \IPS\musicbox\Song::getItemsWithPermission( array( array( 'song_isPlaylist<>?', 1 ) ), 'song_id DESC', \IPS\Settings::i()->mbconf_index_songs ? \IPS\Settings::i()->mbconf_index_songs : 20 );		
		
		/* Online User Location */
		\IPS\Session::i()->setLocation( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), array(), 'loc_musicbox_index' );
		
		/* Breadcrum */
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}			
		
		/* Display */
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('musicbox_titlepage');
		$output = \IPS\Theme::i()->getTemplate( 'browse' )->index( $newPlaylists, $newSongs );
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}
	
	/**
	 * List all rss feeds
	 */
	protected function podcast()
	{
		if ( !\IPS\Settings::i()->mbconf_rss )
		{
			\IPS\Output::i()->error( 'mb_error_podcast_disable', '2BIMMB105/1', 403, '' );
		}
		
		$data = array();
		
		foreach ( \IPS\musicbox\Category::roots() as $cat )
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack( 'mb_rss_title_genre', FALSE, array( 'sprintf' => array( $cat->_title ) ) );
			$url = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=index&do=podcastbygenre&id={$cat->_id}", 'front', 'musicbox_podcastByGenre', array( $cat->name_furl ) );
			$data[$title] = $url;
		}
		
		/* Display */
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title	= \IPS\Member::loggedIn()->language()->addToStack('mb_podcast');
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'browse' )->podcast( $data );		
	}
	
	protected function podcastbygenre()
	{
		if ( !\IPS\Settings::i()->mbconf_rss )
		{
			\IPS\Output::i()->error( 'mb_error_podcast_disable', '2BIMMB105/2', 403, '' );
		}
		
		try
		{
			$category = \IPS\musicbox\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'read' );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB105/3', 404, '' );
		}
						
		$catName = \IPS\Member::loggedIn()->language()->addToStack( \IPS\Member::loggedIn()->language()->outputStack[ $category->_title ]['key'] );
		$rssTitle = \IPS\Settings::i()->board_name . " - " . $catName;
		$rssUrl = (string) $category->url();

		$where = array( array( 'song_file IS NOT NULL AND song_isPlaylist<>?', 1 ) );
		$where[] = array( 'song_cat=?', $category->id );
		foreach( \IPS\musicbox\Song::getItemsWithPermission( $where, 'song_id DESC', 30, 'read' ) as $s )
		{
			$allSongs[] = $s;
		}
		
		if ( \count($allSongs) > 0 )
		{
			$rssImage = $allSongs[0]->image();
		}
						
		$this->createRSS($allSongs, $rssTitle, $rssUrl, $rssImage);
	}
	
	
	protected function podcastbyartist()
	{
		if ( !\IPS\Settings::i()->mbconf_rss )
		{
			\IPS\Output::i()->error( 'mb_error_podcast_disable', '2BIMMB105/4', 403, '' );
		}
		
		try
		{
			$artist = \IPS\musicbox\Artist::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB105/5', 404, '' );
		}
						
		$rssTitle = $artist->name .  " - " . \IPS\Settings::i()->board_name;
		$rssUrl = (string) $artist->url();
		
		$where = array( "song_file IS NOT NULL AND song_isPlaylist!=1 AND ( LOWER(song_singer)=? OR LOWER(song_singer) LIKE CONCAT(?, ',%') OR LOWER(song_singer) LIKE CONCAT('%, ', ?, ',%') OR LOWER(song_singer) LIKE CONCAT('%, ', ?) )", $artist->name, $artist->name, $artist->name, $artist->name );

		foreach( \IPS\musicbox\Song::getItemsWithPermission( $where, 'song_id DESC', 30, 'read' ) as $s )
		{
			$allSongs[] = $s;
		}
		
		if ( $allSongs && \count($allSongs) > 0 )
		{
			$rssImage = $artist->artwork();
		}	
		
		$this->createRSS($allSongs, $rssTitle, $rssUrl, $rssImage);
	}
	
	protected function podcastbyplaylist()
	{
		if ( !\IPS\Settings::i()->mbconf_rss )
		{
			\IPS\Output::i()->error( 'mb_error_podcast_disable', '2BIMMB105/6', 403, '' );
		}
		
		try
		{
			$playlist = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
			if ( $playlist->isPlaylist != 1 || !$playlist->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMMB100/7', 403, '' );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/8', 404, '' );
		}
		
		$rssTitle = $playlist->title .  " - " . \IPS\Settings::i()->board_name;
		$rssUrl = (string) $playlist->url();

		foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', "find_in_set('{$playlist->id}',song_inPlaylist) <> 0", 'song_playlist_pos ASC')  as $s )
		{
			$allSongs[] = \IPS\musicbox\Song::constructFromData($s);
		}
		
		if ( \count($allSongs) > 0 )
		{
			$rssImage = $playlist->thumbnail();
		}
		
		$this->createRSS($allSongs, $rssTitle, $rssUrl, $rssImage, $playlist->author()->name, $playlist->author()->email);
	}	
	
	/* Build the document (check valid in castfeedvalidator.com)*/
	protected function createRSS( $allSongs, $rssTitle, $rssUrl, $rssImage, $ownerName=null, $ownerEmail=null )
	{
		header('Content-Type: text/xml; charset=utf-8', true);
		$xml = new \DOMDocument("1.0", "UTF-8");
		
		$rss = $xml->createElement("rss"); 
		$rss_node = $xml->appendChild($rss); //add RSS element to XML node

		$rss_node->setAttribute("xmlns:atom","http://www.w3.org/2005/Atom");
		$rss_node->setAttribute("xmlns:cc","http://web.resource.org/cc/");
		$rss_node->setAttribute("xmlns:itunes","http://www.itunes.com/dtds/podcast-1.0.dtd");
		$rss_node->setAttribute("xmlns:media","http://search.yahoo.com/mrss/");
		$rss_node->setAttribute("xmlns:rdf","http://www.w3.org/1999/02/22-rdf-syntax-ns#");
		$rss_node->setAttribute("xmlns:feedburner","http://rssnamespace.org/feedburner/ext/1.0");
		$rss_node->setAttribute("version","2.0");			
		$rss_node->setAttribute("xml:lang",\IPS\Member::loggedIn()->language()->bcp47());			

		$channel = $xml->createElement("channel");  
		$channel_node = $rss_node->appendChild($channel);
		
		$channel_atom_link = $xml->createElement("atom:link");  
		$channel_atom_link->setAttribute("href", $rssUrl);
		//$channel_atom_link->setAttribute("rel", "self");
		$channel_atom_link->setAttribute("type", "application/rss+xml");
		$channel_node->appendChild($channel_atom_link); 
		
		$channel_node->appendChild($xml->createElement("title", $rssTitle));
		$channel_node->appendChild($xml->createElement("description", $rssTitle));
		$channel_node->appendChild($xml->createElement("link", $rssUrl));
		$channel_node->appendChild($xml->createElement("language", \IPS\Member::loggedIn()->language()->bcp47())); 
		$channel_node->appendChild($xml->createElement("generator", "PHP DOMDocument"));	

		$itune_category = $channel_node->appendChild($xml->createElement("itunes:category")); 
		$itune_category->setAttribute('text', 'Music Commentary'); 		
		
		$itune_explicit = $channel_node->appendChild($xml->createElement("itunes:explicit", 'clean')); 
		
		$itune_owner = $channel_node->appendChild($xml->createElement("itunes:owner")); 		
		$itune_owner->appendChild($xml->createElement("itunes:name", $ownerName ? $ownerName : \IPS\Settings::i()->board_name)); 
		$itune_owner->appendChild($xml->createElement("itunes:email", $ownerEmail ? $ownerEmail : \IPS\Settings::i()->email_in)); 
					
		$itunes_image_root = $xml->createElement("itunes:image");  
		$itunes_image_root->setAttribute("href", $rssImage);
		$channel_node->appendChild($itunes_image_root); 
		
		$image_node = $channel_node->appendChild($xml->createElement("image")); 		
		$image_node->appendChild($xml->createElement("url", $rssImage)); 
		$image_node->appendChild($xml->createElement("title", $rssTitle)); 
		$image_node->appendChild($xml->createElement("link", $rssUrl)); 
					
		if ( \count($allSongs) > 0 )
		{
			foreach ( $allSongs as $song )
			{
				$item_node = $channel_node->appendChild($xml->createElement("item"));
				$title_node = $item_node->appendChild($xml->createElement("title", $song->title));
				$link_node = $item_node->appendChild($xml->createElement("link", $song->url()));
				
				$guid_link = $xml->createElement("guid", 'musicbox' . $song->id);  
				$guid_link->setAttribute("isPermaLink", "false");
				$guid_node = $item_node->appendChild($guid_link); 

				$description_node = $item_node->appendChild($xml->createElement("description"));  
				$description_contents = $xml->createCDATASection($song->content);  
				$description_node->appendChild($description_contents); 
				
				$enclosure = $item_node->appendChild($xml->createElement('enclosure')); 
				$enclosure->setAttribute('url', $song->musicFile()); 
				$enclosure->setAttribute('type', 'audio/mpeg'); 
				$enclosure->setAttribute('length', $song->filesize); 
				
				$itune_author = $item_node->appendChild($xml->createElement("itunes:author", $song->singers())); 
				
				//$itune_image = $item_node->appendChild($xml->createElement("itunes:image")); 
				//$itune_image->setAttribute('href', $song->thumbnail());
				
				$item_node->appendChild($xml->createElement('pubDate', date('D, d M Y H:i:s O', $song->started))); 					
				
				$itune_explicit_item = $item_node->appendChild($xml->createElement("itunes:explicit", 'clean')); 
			}
		}
		
		$channel_node->appendChild($xml->createElement("media:rating", 'nonadult'));
		
		print $xml->saveXML();
		exit;		
	}
	
	/* Load random song */
	protected function loadRandomSong()
	{	
		\IPS\Session::i()->csrfCheck();
		
		$playing = \intval( \IPS\Request::i()->playingID );
		
		$where[] = array( 'song_isPlaylist!=? AND song_id!=?', 1, $playing );
		
		if ( \IPS\Request::i()->area )
		{
			$area = explode( '_', \IPS\Request::i()->area );
			
			switch( $area[0] )
			{
				case 'playlist':
					$where[] = array( 'song_inPlaylist=?', $area[1] );
				break;
				
				case 'song':
					$where[] = array( 'song_cat=?', $area[1] );
				break;

				case 'artist':
					$where[] = array( "LOWER(song_singer) LIKE CONCAT('%', ?, '%')", urldecode( $area[1] ) );
				break;
				
				case 'favorite':
					$songs = \IPS\musicbox\Song::myFavoriteSongs( $area[1] );
					if (\count($songs) > 0)
					{
						shuffle($songs);
						$song = $songs[0];
					}
				break;				
				
				default:
					break;
			}			
		}
		else
		{
			try
			{
				$pl = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_mem=?', \IPS\Member::loggedIn()->member_id ), 'RAND()' )->first();
			}
			catch ( \UnderflowException $e ){}
			
			if ( $pl['mpl_playlistID'] > 0 )
			{
				$songs = \IPS\musicbox\Song::myFavoriteSongs( $pl['mpl_playlistID'] );
				if ( \count($songs) > 0 )
				{
					shuffle( $songs );
					$song = $songs[0];
				}				
			}
			elseif ( $pl['mpl_songs'] )
			{
				$msongs = explode(',', $pl['mpl_songs']);
				$randID = $msongs[mt_rand(0, \count($msongs) - 1)];
				$where[] = array( 'song_id=?', $randID );
			}
		}
		
		if ( !$song )
		{
			try
			{		
				$song = \IPS\musicbox\Song::constructFromData( \IPS\Db::i()->select( '*', 'musicbox_songs', $where, 'RAND()', 1 )->first() );		
			}			
			catch ( \UnderflowException $e ) {}			
		}

		if ( $song )
		{
			\IPS\Output::i()->json( array(
				'song' 		=> $song->musicFile(),
				'id'		=> $song->id,
				'url'		=> $song->url(),
				'name'		=> $song->title,
				'artist'	=> $song->singer,
				'thumbnail'	=> $song->thumbnail()
			) );
		}
	}
}