<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * playlists
 */
class _playlists extends \IPS\Dispatcher\Controller
{

	protected function manage()
	{	
		\IPS\Output::i()->breadcrumb = array();
		
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}		
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=playlists", 'front', 'musicbox_newestPlaylists' ), \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_newestPlaylists' ) );
		
		if ( isset( \IPS\Request::i()->id ) )
		{
			try
			{
				$this->_category( \IPS\musicbox\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'read' ) );
			}
			catch ( \OutOfRangeException $e )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMMB104/1', 404 );
			}
		}
		else
		{
			$this->_index();
		}
	}
	
	protected function _category( $category )
	{
		/* Build table */
		$catURL = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=playlists&id={$category->id}", 'front', 'musicbox_playlists', $category->name_furl );
		$where = array( array( 'song_isPlaylist=?', 1 ) );
		$table = new \IPS\Helpers\Table\Content( 'IPS\musicbox\Song', $catURL, $where, $category );
		$table->sortOptions = array_merge( $table->sortOptions, array( 'singer' => 'song_singer' ) );	
		if ( \in_array( $table->sortBy, array( 'song_singer', 'song_title' ) ) )
		{
			$table->sortDirection = 'asc';
		}		
		unset ( $table->sortOptions['last_comment'] );
		unset ( $table->sortOptions['num_comments'] );
		unset ( $table->sortOptions['date'] );
		$table->limit = 30;	
		$table->tableTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'musicbox', 'front' ), 'table' );
		$table->rowsTemplate  = array( \IPS\Theme::i()->getTemplate( 'browse', 'musicbox', 'front' ), 'playlistRows' );
		
		/* Online User Location */
		\IPS\Session::i()->setLocation( $catURL, explode( ",", $permissions['perm_view'] ), 'loc_musicbox_viewing_category', array( "musicbox_category_{$category->id}" => TRUE ) );
				
		/* Output */
		$title = $category->_title;
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title		= $title;			
		\IPS\Output::i()->contextualSearchOptions[ \IPS\Member::loggedIn()->language()->addToStack( 'search_contextual_item', FALSE, array( 'sprintf' => array( \IPS\Member::loggedIn()->language()->addToStack( $category->_title ) ) ) ) ] = array( 'type' => 'musicbox_video', 'nodes' => $category->_id );		
		$output	= \IPS\Theme::i()->getTemplate( 'browse' )->genre( $category, (string) $table, $title );
		
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}

	protected function _index()
	{
		/* New playlists */
		$newPlaylists = array();
		$newPlaylists = \IPS\musicbox\Song::getItemsWithPermission( array( array( 'song_isPlaylist=?', 1 ) ), 'song_id DESC', \IPS\Settings::i()->mbconf_index_playlists ? \IPS\Settings::i()->mbconf_index_playlists : 18 );
				
		/* Online User Location */
		$url = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=playlists", 'front', 'musicbox_newestPlaylists' );
		\IPS\Session::i()->setLocation( $url, array(), 'loc_musicbox_viewing_newestPlaylists', array() );		
		
		/* Output */
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		$title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_newestPlaylists');			
		\IPS\Output::i()->title	= $title;
		$output = \IPS\Theme::i()->getTemplate( 'browse' )->genre( $category, $newPlaylists, $title );
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}		
	}
	
	protected function add()
	{
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=post&do=createPlaylist&id=' . \IPS\Request::i()->id ) );
	}	
}