<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * post
 */
class _post extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}
	
	/**
	 *
	 * @return	void
	 */
	protected function postSong()
	{
		$form = new \IPS\Helpers\Form( 'musicbox_select_category', 'continue' );
		$form->class = 'ipsForm_vertical ipsForm_noLabels';
		$form->add( new \IPS\Helpers\Form\Node( 'category',  isset( \IPS\Request::i()->category ) ? \IPS\Request::i()->category : NULL, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=post&do=postSong', 'front', 'musicbox_postSong' ),
			'class'					=> 'IPS\musicbox\Category',
			'permissionCheck'		=> function( $node )
			{
				if ( $node->can( 'view' ) )
				{
					if ( $node->can( 'add' ) )
					{
						return TRUE;
					}
					
					return FALSE;
				}
				
				return NULL;
			},
		) ) );

		if ( $values = $form->values() )
		{
			$url = \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=post&do=doPostSong', 'front', 'musicbox_postSong' )->setQueryString( 'id', $values['category']->_id );
			\IPS\Output::i()->redirect( $url );
		}
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'post' )->categorySelector( $form );
	}
	
	protected function doPostSong()
	{
		/* Get category data */
		$category = NULL;
		
		if ( isset( \IPS\Request::i()->id ) )
		{
			try
			{
				$category = \IPS\musicbox\Category::loadAndCheckPerms( \IPS\Request::i()->id );
			}
			catch ( \OutOfRangeException $e ) { }
		}
		
		if ( !$category )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB103/1', 403, '' );
		}		
		
		/* Form */
		$form = \IPS\musicbox\Song::create( $category );
		$form->hiddenValues['category'] = $category->_id;
		
		/* Display */
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_post' );
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title		= $title;
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}		
		\IPS\Output::i()->breadcrumb[] = array( $category->url(), $category->_title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'post' )->postPage( $category, $form, $title );
	}
	
	protected function createPlaylist()
	{
		$form = new \IPS\Helpers\Form( 'musicbox_select_category', 'continue' );
		$form->class = 'ipsForm_vertical ipsForm_noLabels';
		$form->add( new \IPS\Helpers\Form\Node( 'category',  isset( \IPS\Request::i()->category ) ? \IPS\Request::i()->category : NULL, TRUE, array(
			'url'					=> \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=post&do=createPlaylist', 'front', 'musicbox_createPlaylist' ),
			'class'					=> 'IPS\musicbox\Category',
			'permissionCheck'		=> function( $node )
			{
				if ( $node->can( 'view' ) )
				{
					if ( $node->can( 'plist' ) )
					{
						return TRUE;
					}
					
					return FALSE;
				}
				
				return NULL;
			},
		) ) );

		if ( $values = $form->values() )
		{
			$url = \IPS\Http\Url::internal( 'app=musicbox&module=musicbox&controller=post&do=doCreatePlaylist', 'front', 'musicbox_createPlaylist' )->setQueryString( 'id', $values['category']->_id );
			\IPS\Output::i()->redirect( $url );
		}
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_select_category' );
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'post' )->categorySelector( $form );
	}

	protected function doCreatePlaylist()
	{
		/* Get category data */
		$category = NULL;
		
		if ( isset( \IPS\Request::i()->id ) )
		{
			try
			{
				$category = \IPS\musicbox\Category::loadAndCheckPerms( \IPS\Request::i()->id, 'plist' );
			}
			catch ( \OutOfRangeException $e ) { }
		}
		
		if ( !$category )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB101/2', 403, '' );
		}		
		
		/* Form */
		$form = \IPS\musicbox\Song::create( $category );
		$form->hiddenValues['category'] = $category->_id;
		
		/* Display */	
		$title = \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_postPlaylist' );
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		\IPS\Output::i()->title		= $title;
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}			
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=playlists&id={$category->id}", 'front', 'musicbox_playlists', $category->name_furl ), $category->_title );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );
		\IPS\Output::i()->output	= \IPS\Theme::i()->getTemplate( 'post' )->postPage( $category, $form, $title );
	}	
}