<?php


namespace IPS\musicbox\modules\front\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * profile
 */
class _profile extends \IPS\Dispatcher\Controller
{
	/**
	 * [Content\Controller]	Class
	 */
	protected static $contentModel = 'IPS\musicbox\Artist';
	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{

		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		try
		{
			$artist = \IPS\musicbox\Artist::load( \IPS\Request::i()->id );
			$this->info( $artist );	
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB102/1', 404, '' );
		}
	}
	
	public function info( $artist )
	{
		/* Online User Location */
		\IPS\Session::i()->setLocation( $artist->url(), array(), 'loc_musicbox_viewing_artist', array( $artist->name => FALSE ) );
		
		/* All songs */
		$aname = mb_strtolower( $artist->name );
		$wheresongs = array( "song_isPlaylist!=1 AND LOWER(song_singer) LIKE CONCAT( '%', ?, '%' )", $aname );
		$songs = \IPS\musicbox\Song::getItemsWithPermission( $wheresongs, 'RAND()', 30 );
		
		$whereplaylists = array( "song_isPlaylist=1 AND LOWER(song_singer) LIKE CONCAT( '%', ?, '%' )", $aname );
		$playlists = \IPS\musicbox\Song::getItemsWithPermission( $whereplaylists, 'RAND()', 30 );
		
		/* RSS */
		if ( \IPS\Settings::i()->mbconf_rss )
		{
			$rssTitle = \IPS\Member::loggedIn()->language()->addToStack( 'mb_rss_title_artist', FALSE, array( 'sprintf' => array( $artist->name ) ) );
			$rssUrl = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=index&do=podcastbyartist&id={$artist->id}", 'front', 'musicbox_podcastByArtist', array( $artist->name_furl ) );
			\IPS\Output::i()->rssFeeds[ $rssTitle ] = $rssUrl;
		}
		
		/* Output */
		\IPS\Output::i()->title	= $artist->name;
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ), \IPS\Member::loggedIn()->language()->addToStack( 'artist_list' ) );
		\IPS\Output::i()->breadcrumb[] = array( $artist->url(), $artist->name );
		$output = \IPS\Theme::i()->getTemplate( 'artists' )->artistProfile( $artist, $songs, $playlists );			
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}			
	}
	
	protected function edit()
	{
		if ( ! \IPS\Member::loggedIn()->inGroup( explode(",", \IPS\Settings::i()->mbconf_artists_perm ) ) )
		{
			\IPS\Output::i()->error( 'musicbox_error_artist_noperm', '2BIMMB102/2', 404, '' );
		}
		
		$class = static::$contentModel;
		
		try
		{
			$artist = $class::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB102/3', 404, '' );
		}
		
		$editForm = NULL;
		$editForm = new \IPS\Helpers\Form();
		$editForm->class = 'ipsPad';
		
		$artist->form( $editForm );
		
		if ( $values = $editForm->values() )
		{
			$artist->saveForm( $artist->formatFormValues( $values ) );
			\IPS\Output::i()->redirect( $artist->url() );
		}
		
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ), \IPS\Member::loggedIn()->language()->addToStack( 'artist_list' ) );
		\IPS\Output::i()->breadcrumb[] = array( $artist->url(), $artist->name );		
		\IPS\Output::i()->output = $editForm;
	}
	
	protected function remove()
	{
		if ( ! \IPS\Member::loggedIn()->inGroup( explode(",", \IPS\Settings::i()->mbconf_artists_perm ) ) )
		{
			\IPS\Output::i()->error( 'musicbox_error_artist_noperm', '2BIMMB102/2', 404, '' );
		}
		
		$class = static::$contentModel;
		
		try
		{
			$artist = $class::load( \IPS\Request::i()->id );
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB102/4', 404, '' );
		}	
		
		$artist->delete();
		
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists", 'front', 'musicbox_artists' ), 'mb_artist_deleted' );	
	}
}