<?php

namespace IPS\musicbox\modules\front\musicbox;

/* To prvideo PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * View Video Controller
 */
class _view extends \IPS\Content\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
	
	/**
	 * [Content\Controller]	Class
	 */
	protected static $contentModel = 'IPS\musicbox\Song';

	/**
	 * Init
	 *
	 * @return	void
	 */
	public function execute()
	{			
		try
		{
			$this->song = \IPS\musicbox\Song::load( \IPS\Request::i()->id );
			if ($this->song->isPlaylist != 1)
			{
				$this->containerURL = $this->song->container()->url();
			}
			else
			{
				$this->containerURL = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=playlists&id={$this->song->container()->id}", 'front', 'musicbox_playlists', $this->song->container()->name_furl );
				
				$rssTitle = \IPS\Member::loggedIn()->language()->addToStack( 'mb_podcast_playlist' );
				$rssUrl = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=index&do=podcastbyplaylist&id={$this->song->id}", 'front', 'musicbox_podcastByPlaylist', $this->song->title_furl );		
				\IPS\Output::i()->rssFeeds[$rssTitle] = $rssUrl;
			}
			
			if ( !$this->song->canView( \IPS\Member::loggedIn() ) )
			{
				\IPS\Output::i()->error( 'node_error', '2BIMMB100/1', 403, '' );
			}
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/2', 404, '' );
		}
		
		\IPS\Output::i()->sidebar['enabled'] = FALSE;
		
		parent::execute();
	}
	
	/**
	 * Song
	 *
	 * @return	void
	 */
	protected function manage()
	{
		/* Init */
		parent::manage();
			
		/* Update views (because IPS does not update views in ajax) */
		if ( \IPS\Request::i()->isAjax() )
		{
			\IPS\Db::i()->insert( 'core_view_updates', array(
				'classname'	=> 'IPS\musicbox\Song',
				'id'		=> $this->song->id
			) );		
		}	
					
		/* Info */
		$singer = $this->song->singer ? $this->song->singer : \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_unknownSinger' );
		
		/* Online User Location */
		\IPS\Session::i()->setLocation( $this->song->url(), $this->song->onlineListPermissions(), 'loc_musicbox_viewing_item', array( $this->song->title => FALSE ) );
		
		/* Meta tags */
		\IPS\Output::i()->metaTags['og:image'] = $this->song->thumbnail();

		/* Display */
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}
	
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( $this->containerURL, $this->song->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->song->url(), $this->song->mapped('title') );			
		\IPS\Output::i()->title = $this->song->title . " - " . $singer;
		
		$output = \IPS\Theme::i()->getTemplate( 'view' )->view( $this->song );
		
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $output, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $output;
		}		
	}

	protected function edit()
	{
		if ( !$this->song->canEdit() )
		{
			\IPS\Output::i()->error( 'node_noperm_edit', '2BIMMB100/3', 403, '' );
		}
		
		$class = static::$contentModel;
		
		$formElements = $class::formElements( $this->song, $this->song->container() );		
		$form = new \IPS\Helpers\Form( 'form', isset( \IPS\Member::loggedIn()->language()->words[ $class::$formLangPrefix . '_save' ] ) ? $class::$formLangPrefix . '_save' : 'save' );
		$form->class = 'ipsForm_vertical';
		
		foreach( $formElements as $name => $formContent )
		{
			$form->add( $formContent );
		}
		
		if ( $values = $form->values() )
		{
			$this->song->processForm( $values );
			$this->song->save();
			\IPS\Output::i()->redirect( $this->song->url() );
		}

		
		$title = $this->song->isPlaylist != 1 ? \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_editsong' ) : \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_editplaylist' );
		
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}		
		\IPS\Output::i()->breadcrumb[] = array( $this->containerURL, $this->song->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->song->url(), $this->song->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );		
		\IPS\Output::i()->title = $title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'post' )->postPage( $this->song->container(), $form, $title );

	}	
	
	/*-------------------------------------------------------------------------*/
	// Change author (submitter)
	/*-------------------------------------------------------------------------*/
	public function changeAuthor()
	{
		/* Permission check */
		if ( !$this->song->canChangeAuthor() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '2BIMMB100/4', 403, '' );
		}
		
		/* Build form */
		$form = new \IPS\Helpers\Form;
		$form->add( new \IPS\Helpers\Form\Member( 'author', NULL, TRUE ) );
		$form->class .= 'ipsForm_vertical';

		/* Handle submissions */
		if ( $values = $form->values() )
		{
			$this->song->changeAuthor( $values['author'] );			
			\IPS\Output::i()->redirect( $this->song->url() );
		}
		
		/* Display form */
		\IPS\Output::i()->output = $form->customTemplate( array( \call_user_func_array( array( \IPS\Theme::i(), 'getTemplate' ), array( 'forms', 'core' ) ), 'popupTemplate' ) );
	}	
	
	/*-------------------------------------------------------------------------*/
	// loadSongDataInAjax (Update when change song in playlist)
	/*-------------------------------------------------------------------------*/
	protected function loadSongDataInAjax()
	{
		\IPS\Session::i()->csrfCheck();
		
		# Force using ajax
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/C', 403, '' );
		}
		
		# Update views
		\IPS\Db::i()->insert( 'core_view_updates', array(
			'classname'	=> 'IPS\musicbox\Song',
			'id'		=> $this->song->id
		) );
			
		# Get lyric
		$lyric = \IPS\Theme::i()->getTemplate( 'view' )->lyric( $this->song );
		
		# Get Price
		if( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->mbconf_nexus_on )
		{
			$price = $this->song->price() ? (string) $this->song->price() : \IPS\Member::loggedIn()->language()->addToStack('song_free');
		}
		
		# Get fields
		$fields = $this->song->getSongFields();
		
		# JSON output
		\IPS\Output::i()->json( array( 'lyric' => $lyric, 'price' => $price, 'fields' => $fields ) );
	}
	
	/*-------------------------------------------------------------------------*/
	// Manage Playlist
	/*-------------------------------------------------------------------------*/	
	protected function managePlaylist()
	{
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/5', 404 );
		}
		
		/* Output */
		\IPS\Output::i()->jsFiles	= array_merge( \IPS\Output::i()->jsFiles, \IPS\Output::i()->js( 'jquery/jquery-ui.js', 'core', 'interface' ) );
		$title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_managePlaylist');	
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}			
		\IPS\Output::i()->breadcrumb[] = array( $this->containerURL, $this->song->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->song->url(), $this->song->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );				
		\IPS\Output::i()->title	= $title;
		\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->managePlaylist( $this->song );		
	}
	
	protected function songInPlaylist()
	{
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/6', 404 );
		}
			
		/* Check song */
		if ( \IPS\Request::i()->song > 0 )
		{
			try
			{
				$item = \IPS\musicbox\Song::load( \IPS\Request::i()->song );
			}
			catch ( \OutOfRangeException $e ){}		
		}
		
		if ( !$item->id && \count($this->song->songsInPlaylist()) > \IPS\Settings::i()->mbconf_maxSongs )
		{
			\IPS\Output::i()->error( 'musicbox_error_maxsongs', '2BIMMB100/7', 404 );
		}
		
		$class = static::$contentModel;
		
		$formElements = $class::formElements( $item, $this->song->container() );		
		$form = new \IPS\Helpers\Form( 'form', isset( \IPS\Member::loggedIn()->language()->words[ $class::$formLangPrefix . '_save' ] ) ? $class::$formLangPrefix . '_save' : 'save' );
		$form->class = 'ipsForm_horizontal ipsPad';
		
		foreach( $formElements as $name => $formContent )
		{
			$form->add( $formContent );
		}
		
		if ( $values = $form->values() )
		{
			if ( !$item )
			{
				$item = $class::createFromForm( $values, $this->song->container() );
				$item->inPlaylist = $this->song->id;
				$item->save();
			}
			else
			{
				$item->processForm( $values );
				$item->save();
			}
			
			\IPS\Output::i()->redirect( $this->song->url('managePlaylist') );
		}
		
		/* Output */
		if ( $item->id > 0 )
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_editsong');
		}
		else $title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_add_to_pl');
		\IPS\Output::i()->breadcrumb = array();
		if ( \IPS\Dispatcher::i()->application->default == 0 )
		{
			\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
		}			
		\IPS\Output::i()->breadcrumb[] = array( $this->containerURL, $this->song->container()->_title );
		\IPS\Output::i()->breadcrumb[] = array( $this->song->url(), $this->song->mapped('title') );
		\IPS\Output::i()->breadcrumb[] = array( NULL, $title );		
		\IPS\Output::i()->title	= $title;
		\IPS\Output::i()->output = $form;					
	}
	
	protected function moveInPlaylist()
	{
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_createdpl' ) ) );
			exit;
		}		
		$position = 0;

		if ( \IPS\Request::i()->isAjax() and \IPS\Request::i()->ajax_order )
		{
			$i = 1;
			foreach ( \IPS\Request::i()->ajax_order as $id )
			{
				\IPS\Db::i()->update( 'musicbox_songs', array( 'song_playlist_pos' => $i ), array( 'song_id=?', $id ) );
				$i++;
			}
			\IPS\Output::i()->json( 'OK' );
			return;
		}
		
		\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
	}
	
	protected function delInPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_createdpl' ) ) );
			exit;
		}		
		
		try
		{
			$_song = \IPS\musicbox\Song::load( \IPS\Request::i()->song );
			$_song->delete();
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'node_error' ) ) );
			exit;				
		}
		
		\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'deleted' ) ) );
	}
	
	protected function removeFromPlaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_createdpl' ) ) );
			exit;
		}		
		
		try
		{
			$_song = \IPS\musicbox\Song::load( \IPS\Request::i()->song );
			$pl = explode(",", $_song->inPlaylist);
			$updatePL = array_diff($pl, array($this->song->id));
			$_song->inPlaylist = \count($updatePL) > 0 ? implode(",", $updatePL) : null;		
			$_song->save();
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->json( array( 'type' => 'error', 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'node_error' ) ) );
			exit;				
		}
		
		\IPS\Output::i()->json( array( 'message' => \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_removed_from_pl' ) ) );
	}

	protected function addSongByID()
	{
		\IPS\Session::i()->csrfCheck();
		
		if ( ! $this->song->container()->can('plist') || ! $this->song->canEdit() )
		{
			\IPS\Output::i()->error( 'node_error', '2BIMMB100/8', 404 );
		}		
		
		try
		{
			$_song = \IPS\musicbox\Song::load( \IPS\Request::i()->songID );
			if ( !$_song->inPlaylist )
			{
				$_song->inPlaylist = $this->song->id;
			}
			else
			{
				if ( !\in_array( $this->song->id, explode(",", $_song->inPlaylist) ) )
				{
					$_song->inPlaylist = $_song->inPlaylist . "," . $this->song->id;
				}
			}
			$_song->save();
		}
		catch ( \OutOfRangeException $e )
		{
			\IPS\Output::i()->error( 'musicbox_error_no_song', '2BIMMB100/9', 404 );
		}
		
		\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
	}	
	
	/*-------------------------------------------------------------------------*/
	// Play song
	/*-------------------------------------------------------------------------*/	
	protected function play()
	{
		/* No direct linking check */
		if ( !isset( \IPS\Request::i()->csrfKey ) )
		{
			\IPS\Output::i()->redirect( $this->song->url() );
		}

		\IPS\Session::i()->csrfCheck();
		
		/* Load song */
		if ( \IPS\Settings::i()->mbconf_nexus_on and $this->isPlaylist != 1 and $this->cost and $this->price() and $member->member_id != $this->author()->member_id )
		{
			$songFile = $this->song->canDownload() ? $this->song->file : $this->song->demo;
		}
		else
		{
			$songFile = $this->song->file;
		}
		
		$file = \IPS\File::get( 'musicbox_File', $songFile );

		$this->_play( $file );
	}
	
	protected function _play( \IPS\File $file )
	{
        if ( !$file->filesize() )
        {
            \IPS\Output::i()->error( 'musicbox_no_file', '2BIMMB100/10', 404 );
        }

		/* Print the file, honoring ranges */
		$ranges	= new \IPS\Http\Ranges( $file, 0 );

		/* If using PHP-FPM, close the request so that __destruct tasks are run after data is flushed to the browser
			@see http://www.php.net/manual/en/function.fastcgi-finish-request.php */
		if( function_exists( 'fastcgi_finish_request' ) )
		{
			fastcgi_finish_request();
		}

		exit;
	}	
	
	/*-------------------------------------------------------------------------*/
	// BUY
	/*-------------------------------------------------------------------------*/	
	protected function buy()
	{
		\IPS\Session::i()->csrfCheck();
		
		/* Can we buy? */
		if ( !$this->song->canBuy() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '2BIMMB100/11', 404 );
		}
		
		/* Create the item */		
		$price = $this->song->price();
		$item = new \IPS\musicbox\extensions\nexus\Item\Song( $this->song->title, $price );
		$item->id = $this->song->id;
		try
		{
			$item->tax = \IPS\Settings::i()->mbconf_nexus_tax ? \IPS\nexus\Tax::load( \IPS\Settings::i()->mbconf_nexus_tax ) : NULL;
		}
		catch ( \OutOfRangeException $e ) { }
		if ( \IPS\Settings::i()->mbconf_nexus_gateways )
		{
			$item->paymentMethodIds = explode( ',', \IPS\Settings::i()->mbconf_nexus_gateways );
		}

		$item->payTo = $this->song->author();
		$item->commission = \IPS\Settings::i()->mbconf_nexus_percent;
		if ( $fees = json_decode( \IPS\Settings::i()->mbconf_nexus_transfee, TRUE ) )
		{
			$item->fee = new \IPS\nexus\Money( $fees[ $price->currency ]['amount'], $price->currency );
		}
				
		/* Generate the invoice */
		$invoice = new \IPS\nexus\Invoice;
		$invoice->currency = ( isset( \IPS\Request::i()->cookie['currency'] ) and \in_array( \IPS\Request::i()->cookie['currency'], \IPS\nexus\Money::currencies() ) ) ? \IPS\Request::i()->cookie['currency'] : \IPS\nexus\Customer::loggedIn()->defaultCurrency();
		$invoice->member = \IPS\nexus\Customer::loggedIn();
		$invoice->addItem( $item );
		$invoice->return_uri = "app=musicbox&module=musicbox&controller=view&id={$this->song->id}";
		$invoice->save();
		
		/* Take them to it */
		\IPS\Output::i()->redirect( $invoice->checkoutUrl() );
	}
	
	protected function buyplaylist()
	{
		\IPS\Session::i()->csrfCheck();
		
		/* Can we buy? */
		if ( !$this->song->canBuyPlaylist() )
		{
			\IPS\Output::i()->error( 'no_module_permission', '2BIMMB100/12', 404 );
		}
		
		/* Get Paid Songs */
		$allsongs = $this->song->songsInPlaylist();
		if ( \count($allsongs) > 0 )
		{
			$invoice = new \IPS\nexus\Invoice;
			$invoice->member = \IPS\nexus\Customer::loggedIn();	
			
			foreach( $allsongs as $s )
			{
				if ( $s->cost AND $s->price()->amount > 0 )
				{
					$price = $s->price();
					$item = new \IPS\musicbox\extensions\nexus\Item\Song( $s->title, $price );
					$item->id = $s->id;
					try
					{
						$item->tax = \IPS\Settings::i()->mbconf_nexus_tax ? \IPS\nexus\Tax::load( \IPS\Settings::i()->mbconf_nexus_tax ) : NULL;
					}
					catch ( \OutOfRangeException $e ) { }
					if ( \IPS\Settings::i()->mbconf_nexus_gateways )
					{
						$item->paymentMethodIds = explode( ',', \IPS\Settings::i()->mbconf_nexus_gateways );
					}

					$item->payTo = $s->author();
					$item->commission = \IPS\Settings::i()->mbconf_nexus_percent;
					if ( $fees = json_decode( \IPS\Settings::i()->mbconf_nexus_transfee, TRUE ) )
					{
						$item->fee = new \IPS\nexus\Money( $fees[ $price->currency ]['amount'], $price->currency );
					}					
					$invoice->addItem( $item );
				}
			}
			
			$invoice->return_uri = "app=musicbox&module=musicbox&controller=view&id={$this->song->id}";
			$invoice->save();	
			
			\IPS\Output::i()->redirect( $invoice->checkoutUrl() );
		}
		
		\IPS\Output::i()->redirect( $this->song->url() );
	}
	
	/*-------------------------------------------------------------------------*/
	// Download song
	/*-------------------------------------------------------------------------*/	
	protected function download()
	{
		/* No direct linking check */
		if ( !isset( \IPS\Request::i()->csrfKey ) )
		{
			\IPS\Output::i()->redirect( $this->song->url() );
		}
		\IPS\Session::i()->csrfCheck();
		
		/* Can we download? */
		try
		{
			$this->song->downloadCheck();
		}
		catch ( \DomainException $e )
		{
			\IPS\Output::i()->error( $e->getMessage(), '2BIMMB100/13', 404 );
		}
		
		/* Download */
		if ( $this->song->isPlaylist != 1 )
		{
			// Update Downloads count
			\IPS\Db::i()->update( 'musicbox_songs', array('song_downloads' => $this->song->downloads+1), array( 'song_id=?', $this->song->id ) );
			
			// Custom name
			$format = \IPS\Settings::i()->mbconf_song_name ? \IPS\Settings::i()->mbconf_song_name : "{name}";
			$realname = str_replace("{name}", str_replace("-", " ", $this->song->title_furl), $format);
			$realname = str_replace("{artist}", str_replace(",", "+", $this->song->singer), $realname);
			$fields = $this->song->customFields();
			preg_match_all( "#{field_(.+?)}#", $realname, $match );
			for ( $i = 0 ; $i < \count( $match[0] ) ; $i++ )
			{
				$fieldID = $match[1][$i];
				$realname = str_replace($match[0][$i], $fields['field_' . $fieldID], $realname);
			}			
			$realname = ucwords($realname) . "." . $this->song->fileExt();

			// Real Url
			if( $this->song->external == 1 )
			{
				$songurl = $this->song->file;
				$songfilesize = \IPS\Application::load('musicbox')->remotefileSize($songurl);
			}
			else
			{
				$sf = \IPS\File::get( 'musicbox_File', $this->song->file );
				$songurl = (string) $sf->url;
				$songfilesize = mb_strpos( $songurl, 'http' ) === false ? str_replace( '//', \IPS\Request::i()->url()->data['scheme'] . '://', $sf->filesize ) : $sf->filesize;
			}

			// Get file now
			// $songfilesize = $this->song->external != 1 ? $this->song->filesize : \IPS\Application::load('musicbox')->remotefileSize($this->song->file);
			// header('Content-Description: File Transfer');
			// header('Content-Type: application/octet-stream');
			// header('Content-Disposition: attachment; filename='.$realname);
			// header('Content-Transfer-Encoding: binary');
			// header('Expires: 0');
			// header('Cache-Control: must-revalidate');
			// header('Pragma: public');
			// header('Content-Length: ' . $songfilesize);
			// ob_clean();
			// flush();
			// @readfile($songurl);
			// exit;
			
			if ( $this->song->external == 1 )
			{
				\IPS\Output::i()->redirect( $songurl );
			}
			else
			{
				$file = \IPS\File::get( 'musicbox_File', $this->song->file );
				$file->originalFilename = $realname;
				$this->_download( $file );
			}
		}
		else
		{
			$title = \IPS\Member::loggedIn()->language()->addToStack('musicbox_download');
			\IPS\Output::i()->breadcrumb = array();
			if ( \IPS\Dispatcher::i()->application->default == 0 )
			{
				\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( 'app=musicbox', 'front', 'musicbox' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__musicbox_musicbox' ) );			
			}				
			\IPS\Output::i()->breadcrumb[] = array( $this->containerURL, $this->song->container()->_title );
			\IPS\Output::i()->breadcrumb[] = array( $this->song->url(), $this->song->mapped('title') );
			\IPS\Output::i()->breadcrumb[] = array( NULL, $title );				
			\IPS\Output::i()->title	= $title;
			\IPS\Output::i()->output = \IPS\Theme::i()->getTemplate( 'view' )->download( $this->song );			
		}
	}
	
	protected function _download( \IPS\File $file )
	{
        if ( !$file->filesize() )
        {
            \IPS\Output::i()->error( 'mb_no_file', '2BIMMB100/14', 404 );
        }

		/* If it's an AWS file just redirect to it */
		if ( $file instanceof \IPS\File\Amazon )
		{
			\IPS\Output::i()->redirect( $file->generateTemporaryDownloadUrl() );
		}

		/* Print the file, honoring ranges */
		$ranges	= new \IPS\Http\Ranges( $file, 0 );

		/* If using PHP-FPM, close the request so that __destruct tasks are run after data is flushed to the browser
			@see http://www.php.net/manual/en/function.fastcgi-finish-request.php */
		if( function_exists( 'fastcgi_finish_request' ) )
		{
			fastcgi_finish_request();
		}

		exit;
	}	
	
}