<?php


namespace IPS\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Artist Node
 */
class _Artist extends \IPS\Node\Model implements \IPS\Node\Ratings
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'musicbox_artists';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'artist_';
	
	/**
	 * @brief	[ActiveRecord] ID Database Column
	 */
	public static $databaseColumnId = 'id';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'name';
	
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'musicbox_artists';
	
	/**
	 * @brief	[Node] Subnode class
	 */
	public static $subnodeClass = NULL;
	
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = TRUE;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 */
	protected static $restrictions = array(
		'app'		=> 'musicbox',
		'module'	=> 'musicbox',
		'prefix'	=> 'artists',
		'all'		=> 'artists_manage',
	);

	public static $titleLangPrefix = NULL;
	
	protected function get__title()
	{
		return $this->name;
	}
	
	/**
	 * @brief	[Node] By mapping appropriate columns (rating_average and/or rating_total + rating_hits) allows to cache rating values
	 */
	public static $ratingColumnMap	= array(
		'rating_average'	=> 'rating_average',
		'rating_total'		=> 'rating_total',
		'rating_hits'		=> 'rating_count',
	);
	
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column = 'artist_name';
		}

		return parent::search( $column, $query, 'artist_name ASC', $where );
	}
	
	/**
	 * Get URL
	 *
	 * @return	\IPS\Http\Url
	 */
	public function url()
	{
		$url = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=profile&id={$this->id}", 'front', 'musicbox_artistProfile', $this->name_furl );
		return $url;
	}	

	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->add( new \IPS\Helpers\Form\Text( 'artist_name', $this->id ? $this->name : "", TRUE, array() ) );
		$form->add( new \IPS\Helpers\Form\Upload( 'artist_avatar', ( $this->id AND $this->avatar ) ? \IPS\File::get( 'musicbox_Artist', $this->avatar ) : NULL, FALSE, array( 
			'storageExtension' 	=> 'musicbox_Artist', 
			'image' 			=> TRUE,
			'maxFileSize'		=> 1, 
		) ) );

		$form->add( new \IPS\Helpers\Form\Editor( 'artist_profile', $this ? $this->profile : NULL, FALSE, array( 
			'app' 			=> 'musicbox', 
			'key' 			=> 'Artist', 
			'autoSaveKey' 	=> 'musicbox-new-artist', 
			'attachIds' 	=> ( $this === NULL ? NULL : array( $this->id, NULL, 'artist' ) ) 
		) ) );
	}
	

	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
	{
		if ( !$this->id )
		{
			$this->save();
			\IPS\File::claimAttachments( 'musicbox-new-artist', $this->id, NULL, 'artist' );	
		}
		
		$this->name = $values['artist_name'];
		$this->name_furl = \IPS\Http\Url::seoTitle( $values['artist_name'] );
		unset( $values[ 'artist_name' ] );

		$this->avatar = (string) $values['artist_avatar'];
		$this->avatar_thumb = $values['artist_avatar'] ? (string) $values['artist_avatar']->thumbnail('musicbox_Artist', 250, 250, true) : null;
		unset( $values[ 'artist_avatar' ] );
		
		return $values;
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		\IPS\File::unclaimAttachments( 'musicbox_Artist', $this->id );
		
		if ( $this->avatar )
		{
			try
			{			
				\IPS\File::get( 'musicbox_Artist', $this->avatar )->delete();
				\IPS\File::get( 'musicbox_Artist', $this->avatar_thumb )->delete();
			}
			catch( \Exception $e ){}
		}
		
		parent::delete();
	}
	
	/*-------------------------------------------------------------------------*/
	// get avatar
	/*-------------------------------------------------------------------------*/
	public function avatar()
	{
		return $this->avatar_thumb ? (string) \IPS\File::get( 'musicbox_Artist', $this->avatar_thumb )->url : (string) \IPS\Theme::i()->resource( 'default_photo.png', 'core', 'global'	);
	}
	
	public function artwork()
	{
		return $this->avatar ? (string) \IPS\File::get( 'musicbox_Artist', $this->avatar )->url : (string) \IPS\Theme::i()->resource( 'default_photo.png', 'core', 'global'	);
	}	
}