<?php

namespace IPS\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Song
 */
class _Song extends \IPS\Content\Item implements
\IPS\Content\Lockable,
\IPS\Content\Tags,
\IPS\Content\Permissions,
\IPS\Content\Ratings,
\IPS\Content\Followable,
\IPS\Content\ReadMarkers,
\IPS\Content\Pinnable,
\IPS\Content\Hideable,
\IPS\Content\Shareable,
\IPS\Content\Searchable,
\IPS\Content\Embeddable
{
	use \IPS\Content\Reactable, \IPS\Content\Reportable, \IPS\Content\ViewUpdates;
	
	/**
	 * @brief	Application
	 */
	public static $application = 'musicbox';
	
	/**
	 * @brief	Module
	 */
	public static $module = 'musicbox';
	
	/**
	 * @brief	Database Table
	 */
	public static $databaseTable = 'musicbox_songs';
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = 'song_';
	
	/**
	 * @brief	Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	Node Class
	 */
	public static $containerNodeClass = 'IPS\musicbox\Category';
	
	/**
	 * @brief	[Content\Item]	Comment Class
	 */
	public static $commentClass = 'IPS\musicbox\Song\Comment';
	
	/**
	 * Set custom posts per page setting
	 *
	 * @return int
	 */
	public static function getCommentsPerPage()
	{
		return 15;
	}	

	/**
	 * @brief	Database Column Map
	 */
	public static $databaseColumnMap = array(
		'container'				=> 'cat',
		'author'				=> 'author',
		'views'					=> 'views',
		'title'					=> 'title',
		'content'				=> 'content',
		'num_comments'			=> 'comments',
		'unapproved_comments'	=> 'unapproved_comments',
		'hidden_comments' 		=> 'hidden_comments',		
		'last_comment'			=> 'last_comment',	
		'last_comment_by'		=> 'last_comment_mid',
		'date'					=> 'started',
		'updated'				=> 'updated',
		'rating'				=> 'rating',		
		'rating_average'		=> 'rating_average',
		'rating_total'			=> 'rating_total',	
		'locked'				=> 'locked',
		'approved'				=> 'open',
		'approved_by'			=> 'approver',
		'approved_date'			=> 'approvedon',
		'pinned'				=> 'pinned',
		'ip_address'			=> 'ipaddress',
		'singer'				=> 'singer',
	);
	
	/**
	 * @brief	Hover preview
	 */
	public $tableHoverUrl = FALSE;
	
	/**
	 * @brief	Title
	 */
	public static $title = 'musicbox_song';
	
	/**
	 * @brief	Icon
	 */
	public static $icon = 'music';
	
	/**
	 * @brief	Form Lang Prefix
	 */
	public static $formLangPrefix = 'musicbox_';
	
	
	/**
	 * @brief	Reputation Type
	 */
	public static $reputationType = 'song_id';
	
	/**
	 * @brief	[Content]	Key for hide reasons
	 */
	public static $hideLogKey = 'musicbox-song';

	
	/**
	 * Columns needed to query for search result / stream view
	 *
	 * @return	array
	 */
	public static function basicDataColumns()
	{
		$return = parent::basicDataColumns();
		$return[] = 'song_views';
		$return[] = 'song_content';
		$return[] = 'song_singer';
		$return[] = 'song_isPlaylist';
		$return[] = 'song_playlist_image';
		$return[] = 'song_playlist_image_thumb';
		return $return;
	}
	
	/**
	 * Reaction type
	 *
	 * @return	string
	 */
	public static function reactionType()
	{
		return 'song_id';
	}
	
	/**
	 * Set title
	 *
	 * @param	string	$title	Title
	 * @return	void
	 */
	public function set_title( $title )
	{
		$this->_data['title'] = $title;
		$this->_data['title_furl'] = \IPS\Http\Url::seoTitle( $title );
	}	

	/**
	 * @brief	Cached URLs
	 */
	protected $_url	= array();
	
	/**
	 * @brief	URL Base
	 */
	public static $urlBase = 'app=musicbox&module=musicbox&controller=view&id=';
	
	/**
	 * @brief	URL Base
	 */
	public static $urlTemplate = 'musicbox_playSong';
	
	/**
	 * @brief	SEO Title Column
	 */
	public static $seoTitleColumn = 'title_furl';
	
	/**
	 * Get template for content tables
	 *
	 * @return	callable
	 */
	public static function contentTableTemplate()
	{
		return array( \IPS\Theme::i()->getTemplate( 'browse', 'musicbox', 'front' ), 'songRows' );
	}	
	
	/**
	 * Can view?
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for or NULL for the currently logged in member
	 * @return	bool
	 */
	public function canView( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();
		return parent::canView( $member );
	}
	
	/**
	 * Can edit?
	 * Authors can always edit their own files
	 *
	 * @param	\IPS\Member|NULL	$member	The member to check for (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canEdit( $member=NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		if ( !$member->member_id )
		{
			return FALSE;
		}

		return ( $member->member_id == $this->author()->member_id ) or parent::canEdit( $member );
	}
	
	/**
	 * Can Rate?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @return	bool
	 * @throws	\BadMethodCallException
	 */
	public function canRate( \IPS\Member $member = NULL )
	{
		return $this->container()->bitoptions['rating'] and parent::canRate( $member );
	}	

	/**
	 * Can change author?
	 *
	 * @param	\IPS\Member\NULL	$member	The member (NULL for currently logged in member)
	 * @return	bool
	 */
	public function canChangeAuthor( \IPS\Member $member = NULL )
	{
		return static::modPermission( 'edit', $member, $this->container() );
	}

	/**
	 * Change Author
	 *
	 * @param	\IPS\Member	$newAuthor	The new author
	 * @return	void
	 */
	public function changeAuthor( \IPS\Member $newAuthor, $log=TRUE )
	{
		parent::changeAuthor( $newAuthor );
	}
	
	/**
	 * Get snippet HTML for search result display
	 *
	 * @param	array		$indexData		Data from the search index
	 * @param	array		$authorData		Basic data about the author. Only includes columns returned by \IPS\Member::columnsForPhoto()
	 * @param	array		$itemData		Basic data about the item. Only includes columns returned by item::basicDataColumns()
	 * @param	array|NULL	$containerData	Basic data about the author. Only includes columns returned by container::basicDataColumns()
	 * @param	array		$reputationData	Array of people who have given reputation and the reputation they gave
	 * @param	int|NULL	$reviewRating	If this is a review, the rating
	 * @param	string		$view			'expanded' or 'condensed'
	 * @return	callable
	 */
	public static function searchResultSnippet( array $indexData, array $authorData, array $itemData, array $containerData = NULL, array $reputationData, $reviewRating, $view )
	{
		$song = \IPS\musicbox\Song::constructFromData( $itemData );
		$url = \IPS\Http\Url::internal( static::$urlBase . $indexData['index_item_id'], 'front', static::$urlTemplate, \IPS\Http\Url::seoTitle( $indexData['index_title'] ?: $itemData[ static::$databasePrefix . static::$databaseColumnMap['title'] ] ) );
		return \IPS\Theme::i()->getTemplate( 'global', 'musicbox', 'front' )->searchResultSongSnippet( $song, $url, $view == 'condensed' );
	}
	
	/**
	 * Get image for embed
	 *
	 * @return	\IPS\File|NULL
	 */
	public function embedImage()
	{	
		return $this->thumbnail();
	}	

	/**
	 * Returns the content images
	 *
	 * @param	int|null	$limit	Number of attachments to fetch, or NULL for all
	 *
	 * @return	array|NULL	If array, then array( 'core_Attachment' => 'month_x/foo.gif', ... );
	 * @throws	\BadMethodCallException
	 */
	public function contentImages( $limit = NULL, $ignorePermissions = false )
	{
		if ( $this->isPlaylist == 1 && $this->playlist_image_thumb )
		{
			$images[] = array( 'musicbox_Playlist' => (string) \IPS\File::get( 'musicbox_Playlist', $this->playlist_image_thumb )->url );
		}
		if ( $this->singer )
		{
			foreach( explode(",", trim($this->singer) . ",") as $name )
			{
				try
				{
					$artist = \IPS\Db::i()->select( '*', 'musicbox_artists', array( 'artist_name=?', $name ) )->first();
				}
				catch ( \UnderflowException $e ){}
				if ( $artist['artist_avatar_thumb'] )
				{
					$images[] = array( 'musicbox_Artist' => (string) \IPS\File::get( 'musicbox_Playlist', $artist['artist_avatar_thumb'] )->url );
					break;
				}
			}
		}

		return ( $images && \count( $images ) ) ? $images : NULL;
	}
	
	/**
	 * Get content for embed
	 *
	 * @param	array	$params	Additional parameters to add to URL
	 * @return	string
	 */
	public function embedContent( $params )
	{	
		return \IPS\Theme::i()->getTemplate( 'embed', 'musicbox' )->embedContent( $this, $this->url()->setQueryString( $params ) );
	}
	
	/**
	 * Get items with permisison check
	 *
	 * @param	array		$where				Where clause
	 * @param	string		$order				MySQL ORDER BY clause (NULL to order by date)
	 * @param	int|array	$limit				Limit clause
	 * @param	string		$permissionKey		A key which has a value in the permission map (either of the container or of this class) matching a column ID in core_permission_index
	 * @param	bool|NULL	$includeHiddenItems	Include hidden files? Boolean or NULL to detect if currently logged member has permission
	 * @param	int			$queryFlags			Select bitwise flags
	 * @param	\IPS\Member	$member				The member (NULL to use currently logged in member)
	 * @param	bool		$joinContainer		If true, will join container data (set to TRUE if your $where clause depends on this data)
	 * @param	bool		$joinComments		If true, will join comment data (set to TRUE if your $where clause depends on this data)
	 * @param	bool		$joinReviews		If true, will join review data (set to TRUE if your $where clause depends on this data)
	 * @return	\IPS\Patterns\ActiveRecordIterator|int
	 */
	public static function getItemsWithPermission( $where=array(), $order=NULL, $limit=10, $permissionKey='read', $includeHiddenItems=\IPS\Content\Hideable::FILTER_AUTOMATIC, $queryFlags=0, \IPS\Member $member=NULL, $joinContainer=FALSE, $joinComments=FALSE, $joinReviews=FALSE, $countOnly=FALSE, $joins=NULL, $skipPermission=FALSE, $joinTags=TRUE, $joinAuthor=TRUE, $joinLastCommenter=TRUE, $showMovedLinks=FALSE, $location=NULL )		
	{
		$member = $member ?: \IPS\Member::loggedIn();
		if ( !$member->isAdmin() )
		{
			$where[] = array( 'musicbox_categories.copen=1' );
			$joinContainer = TRUE;
		}
				
		return parent::getItemsWithPermission( $where, $order, $limit, $permissionKey, $includeHiddenItems, $queryFlags, $member, $joinContainer, $joinComments, $joinReviews, $countOnly, $joins, $skipPermission, $joinTags, $joinAuthor, $joinLastCommenter, $showMovedLinks, $location );
	}
	
	/**
	 * Should new items be moderated?
	 *
	 * @param	\IPS\Member		$member		The member posting
	 * @param	\IPS\Node\Model	$container	The container
	 * @return	bool
	 */
	public static function moderateNewItems( \IPS\Member $member, \IPS\Node\Model $container = NULL, $considerPostBeforeRegistering = FALSE )
	{
		if ( $container and $container->bitoptions['moderation'] and !$member->group['g_avoid_q'] )
		{
			return TRUE;
		}
		
		return parent::moderateNewItems( $member, $container );
	}
	
	
	/**
	 * Should new comments be moderated?
	 *
	 * @param	\IPS\Member	$member	The member posting
	 * @return	bool
	 */
	public function moderateNewComments( \IPS\Member $member, $considerPostBeforeRegistering = FALSE )
	{
		$commentClass = static::$commentClass;
		return ( $this->container()->bitoptions['comment_moderation'] and !$member->group['g_avoid_q'] ) or parent::moderateNewComments( $member );
	}

	/**
	 * Get elements for add/edit form
	 *
	 * @param	\IPS\Content\Item|NULL	$item		The current item if editing or NULL if creating
	 * @param	int						$container	Container (e.g. forum) ID, if appropriate
	 * @return	array
	 */
	public static function formElements( $item=NULL, \IPS\Node\Model $container=NULL )
	{
		$return = parent::formElements( $item, $container );

		/* Song title */
		$return['title'] = new \IPS\Helpers\Form\Text( 'musicbox_title', $item ? $item->title : "", TRUE, array( 'maxLength' => 255 ) );
		if ( $item && $item->inPlaylist )
		{
			if ( \count( $item->playlistInfo() ) > 0 )
			{
				foreach( $item->playlistInfo() as $info )
				{
					$pList[] = "<a href='{$info->url()}' target=_blank>" . $info->title . "</a>";
				}
				
				\IPS\Member::loggedIn()->language()->words['musicbox_title_desc'] = \IPS\Member::loggedIn()->language()->addToStack('musicbox_inpl') . implode(", ", $pList); 				
			}
		}
		
		/* Singer(s) */
		$allArtists = array();
		try
		{
			foreach( \IPS\musicbox\Artist::roots() as $artist )
			{
				$allArtists[] = $artist->name;
			}
		}
		catch( \OutOfRangeException $e ) {}
		
		$return['singer'] = new \IPS\Helpers\Form\Text( 'musicbox_singer', $item ? $item->singer : "", FALSE, array( 'autocomplete' => array( 'source' => $allArtists, 'unique' => true, 'freeChoice' => true, 'minimized' => false ) ) );
		
		// Playlist
		if ( \IPS\Request::i()->do == 'doCreatePlaylist' || $item->isPlaylist == 1 )
		{
			$return['thumbnail'] = new \IPS\Helpers\Form\Upload( 'musicbox_playlist_image', ( $item AND $item->playlist_image ) ? \IPS\File::get( 'musicbox_Playlist', $item->playlist_image ) : NULL, FALSE, array( 
				'storageExtension' 	=> 'musicbox_Playlist', 
				'image'				=> true,
				'maxFileSize'		=> \IPS\Settings::i()->mbconf_maxfilesize > 0 ? \IPS\Settings::i()->mbconf_maxfilesize : 20, 
			) );	
		}
		
		// Song
		else
		{
			$return['image'] = new \IPS\Helpers\Form\Upload( 'musicbox_image', ( $item AND $item->image ) ? \IPS\File::get( 'musicbox_Image', $item->image ) : NULL, FALSE, array( 
				'storageExtension' 	=> 'musicbox_Image', 
				'image'				=> true,
				'maxFileSize'		=> \IPS\Settings::i()->mbconf_maxfilesize > 0 ? \IPS\Settings::i()->mbconf_maxfilesize : 20, 
			) );	
			
			
			$options['upload'] = 'musicbox_file_upload';
			$options['external'] = 'musicbox_file_external';
			$toggles = array( 'upload' => array( 'mb_upload' ), 'external' => array( 'mb_external' ) );
			
			$return['fileFrom'] = new \IPS\Helpers\Form\Select( 'musicbox_file_from', $item->external == 1 ? 'external' : 'upload', FALSE, array( 'options' => $options, 'toggles' => $toggles ) );

			$return['file'] = new \IPS\Helpers\Form\Upload( 'musicbox_file', ( $item AND $item->file AND !$item->external ) ? \IPS\File::get( 'musicbox_File', $item->file ) : NULL, TRUE, array( 
				'storageExtension' 	=> 'musicbox_File', 
				'allowedFileTypes'	=> array( 'mp3', 'wma', 'm4a', 'wav' ),
				'maxFileSize'		=> \IPS\Settings::i()->mbconf_maxfilesize ? \IPS\Settings::i()->mbconf_maxfilesize : 20, 
			), NULL, NULL, NULL, 'mb_upload' );		
			
			$return['externalURL'] = new \IPS\Helpers\Form\Text( 'musicbox_external', $item && $item->external == 1 ? $item->file : NULL, FALSE, array(), NULL, NULL, NULL, 'mb_external' );	
			
			if ( $container->bitoptions['lyric'] )
			{
				$return['content']	= new \IPS\Helpers\Form\Editor( 'musicbox_content', $item ? $item->content : NULL, FALSE, array( 
					'app' 			=> 'musicbox', 
					'key' 			=> 'Musicbox', 
					'autoSaveKey' 	=> 'musicbox-new-song', 
					'attachIds' 	=> ( $item === NULL ? NULL : array( $item->id, NULL, 'content' ) ) 
				) );
			}
			
			/* Custom Fields */
			$customFieldValues = $item ? $item->customFields() : array();
			foreach ( $container->cfields as $k => $field )
			{
				$return[] = $field->buildHelper( isset( $customFieldValues[ "field_{$k}" ] ) ? $customFieldValues[ "field_{$k}" ] : NULL );
			}		
			
			/* Nexus Integration */
			if( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->mbconf_nexus_on and \IPS\Member::loggedIn()->inGroup( explode(",", \IPS\Settings::i()->mbconf_nexus_can_add) ) )
			{
				$options = array(
					'free'		=> 'song_free',
					'paid'		=> 'song_paid',
				);

				$return['song_cost_type'] = new \IPS\Helpers\Form\Radio( 'song_cost_type', $item && $item->cost ? 'paid' : 'free', TRUE, array(
					'options'	=> $options,
					'toggles'	=> array( 'paid' => array( 'song_cost', 'music_demo' ) )
				) );
				
				$commissionBlurb = NULL;
				$fees = NULL;
				if ( $_fees = json_decode( \IPS\Settings::i()->mbconf_nexus_transfee, TRUE ) )
				{
					$fees = array();
					foreach ( $_fees as $fee )
					{
						$fees[] = (string) ( new \IPS\nexus\Money( $fee['amount'], $fee['currency'] ) );
					}
					$fees = \IPS\Member::loggedIn()->language()->formatList( $fees, \IPS\Member::loggedIn()->language()->get('or_list_format') );
				}
				if ( \IPS\Settings::i()->mbconf_nexus_percent and $fees )
				{
					$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack( 'song_cost_desc_both', FALSE, array( 'sprintf' => array( \IPS\Settings::i()->mbconf_nexus_percent, $fees ) ) );
				}
				elseif ( \IPS\Settings::i()->mbconf_nexus_percent )
				{
					$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack('song_cost_desc_percent', FALSE, array( 'sprintf' => \IPS\Settings::i()->mbconf_nexus_percent ) );
				}
				elseif ( $fees )
				{
					$commissionBlurb = \IPS\Member::loggedIn()->language()->addToStack('song_cost_desc_fee', FALSE, array( 'sprintf' => $fees ) );
				}
				
				\IPS\Member::loggedIn()->language()->words['song_cost_desc'] = $commissionBlurb;			
				$return['song_cost'] = new \IPS\nexus\Form\Money( 'song_cost', $item ? json_decode( $item->cost, TRUE ) : array(), NULL, array(), NULL, NULL, NULL, 'song_cost' );		
				
				// if ( \is_callable( 'exec' ) AND mb_stripos( ini_get( 'disable_functions' ), 'exec' ) === false )
				// {
					// $ffmpeg = trim( \exec("which ffmpeg") );
					// if (empty($ffmpeg))
					// {
						$return['music_demo'] = new \IPS\Helpers\Form\Upload( 'music_demo', ( $item AND $item->demo ) ? \IPS\File::get( 'musicbox_File', $item->demo ) : NULL, TRUE, array( 
							'storageExtension' 	=> 'musicbox_File', 
							'allowedFileTypes'	=> array( 'mp3', 'wma', 'm4a' ),
							'maxFileSize'		=> \IPS\Settings::i()->mbconf_maxfilesize ? \IPS\Settings::i()->mbconf_maxfilesize : 20, 
						), NULL, NULL, NULL, 'music_demo' );	
					// }
				// }
			}			
		}
				
		/* Build the topic state toggles */
		if ( \IPS\Request::i()->do != 'songInPlaylist' )
		{
			$options = array();
			$toggles = array();

			if ( static::modPermission( 'hide', NULL, $container ) )
			{
				$options['hide'] = 'musicbox_post_hidden';
				$toggles['hide'] = array( 'musicbox_post_hidden' );
				if ( $item AND $item->open != 1 )
				{
					$values[] = 'hide';
				}			
			}

			if ( \count( $options ) or \count( $toggles ) )
			{
				$return['song_afterPosting'] = new \IPS\Helpers\Form\CheckboxSet( 'song_afterPosting', $values ? $values : array(), FALSE, array(
					'options' 	=> $options,
					'toggles'	=> $toggles,
					'multiple'	=> TRUE
				) );	
			}		
		}
		/* Done */
		return $return;
	}	

	/**
	 * Process create/edit form
	 *
	 * @param	array				$values	Values from form
	 * @return	void
	 */
	public function processForm( $values )
	{		
		parent::processForm( $values );
		
		$this->singer = implode( ", ", $values['musicbox_singer'] );
		
		foreach( $values['musicbox_singer'] as $_s )
		{
			$aname = mb_strtolower( $_s );
			
			try
			{
				\IPS\Db::i()->select( '*', 'musicbox_artists', array( "LOWER(artist_name)=?", $aname ), 'artist_name DESC', 1 )->first();	
			}
			catch ( \UnderflowException $e )
			{				
				\IPS\Db::i()->insert( 'musicbox_artists', array( 	
					'artist_name' 		=> ucfirst($aname),
					'artist_name_furl'	=> \IPS\Http\Url::seoTitle( $aname )
				) );
			}			
		}
		
		if ( \IPS\Request::i()->do == 'doCreatePlaylist' || $this->isPlaylist == 1 )
		{		
			$this->isPlaylist = 1;
			$this->playlist_image = (string) $values['musicbox_playlist_image'];
			$this->playlist_image_thumb = $values['musicbox_playlist_image'] ? (string) $values['musicbox_playlist_image']->thumbnail('musicbox_Playlist', 250, 250, true) : null;		
		}
		else
		{
			$this->external = $values['musicbox_file_from'] == 'upload' ? 0 : 1;
			$this->file = $values['musicbox_file_from'] == 'upload' ? (string) $values['musicbox_file'] : $values['musicbox_external'];
			$this->filesize = $values['musicbox_file_from'] == 'upload' ? $values['musicbox_file']->filesize() : \IPS\Application::load('musicbox')->remotefileSize($values['musicbox_external']);
			
			$this->image = (string) $values['musicbox_image'];		
			$this->image_thumb = $values['musicbox_image'] ? (string) $values['musicbox_image']->thumbnail('musicbox_Image', 250, 250, true) : null;		
			
			$this->content = $values['musicbox_content'];

			/* Save ccfields */
			$cfields = array();
			foreach ( $this->container()->cfields as $field )
			{
				$helper							 = $field->buildHelper();
				$cfields[ "field_{$field->id}" ] = $helper::stringValue( $values[ "musicbox_field_{$field->id}" ] );
				
				if ( $helper instanceof \IPS\Helpers\Form\Editor )
				{
					$field->claimAttachments( $this->id );
				}
			}
			
			if ( !empty( $cfields ) )
			{
				\IPS\Db::i()->insert( 'musicbox_ccontent', array_merge( array( 'song_id' => $this->id, 'updated' => time() ), $cfields ), TRUE );
			}
		
			/* Save paid song */
			if( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->mbconf_nexus_on and \IPS\Member::loggedIn()->inGroup( explode(",", \IPS\Settings::i()->mbconf_nexus_can_add) ) )
			{
				if ( $values['song_cost_type'] == 'paid' )
				{
					$this->cost = json_encode( $values['song_cost'] );
					
					if (isset($values['music_demo']))
					{
						$this->demo = (string) $values['music_demo'];	
					}
					else
					{
						// $uploadDir = str_replace('\\', '/', $values['musicbox_file']->configuration['dir']) . '/';
						// $containerDir = $values['musicbox_file']->container . '/';
						// $file = $uploadDir . (string) $values['musicbox_file'];
						// $newName = basename( $values['musicbox_file']->originalFilename ) . ".demo." . mb_substr(mb_strrchr($values['musicbox_file']->originalFilename,'.'),1);
						// $demo = $uploadDir . $containerDir . $newName;

						// \exec( "ffmpeg -t 30 -i {$file} -acodec copy {$demo}" );
						// $this->demo = $containerDir . $newName;								
					}
				}
				else
				{
					$this->cost = NULL;
				}
			}			
		}
		
		/* Moderator actions */
		if ( isset( $values['song_afterPosting'] ) )
		{
			$this->open = \in_array( 'hide', $values['song_afterPosting'] ) ? 0 : 1;
		}	

		/* Save */
		$this->save();
	}

	/**
	 * Process created object AFTER the object has been created
	 *
	 * @param	\IPS\Content\Comment|NULL	$comment	The first comment
	 * @param	array						$values		Values from form
	 * @return	void
	 */	
	protected function processAfterCreate( $comment, $values )
	{
		\IPS\File::claimAttachments( 'musicbox-new-song', $this->id, NULL, 'content' );			
	
		parent::processAfterCreate( $comment, $values );
	}	
	
	/**
	 * Get custom field values
	 * @return	array
	 */
	public function customFields()
	{
		$return = array();
		$fields = $this->container()->cfields;

		try
		{
			$data = \IPS\Db::i()->select( '*', 'musicbox_ccontent', array( 'song_id=?', $this->id ) )->first();
			
			foreach ( $data as $k => $v )
			{
				if ( array_key_exists( str_replace( 'field_', '', $k ), $fields ) )
				{
					$return[ $k ] = $v;
				}
			}
		}
		catch( \UnderflowException $e ){}
		
		return $return;
	}
		
	public function getSongFields()
	{
		/* Custom Field Formatting */
		$cfields	= array();
		$fields		= $this->customFields();

		foreach ( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( 'mbf.*', array( 'musicbox_cfields', 'mbf' ), NULL, 'mbf.cf_position' ), 'IPS\musicbox\Field' ) as $field )
		{
			if( array_key_exists( 'field_' . $field->id, $this->customFields() ) )
			{
				if ( $fields[ 'field_' . $field->id ] !== null AND $fields[ 'field_' . $field->id ] !== '' )
				{
					$cfields[ 'field_' . $field->id ] = $field->displayValue( $fields[ 'field_' . $field->id ] );
				}
			}
		}
		return \IPS\Theme::i()->getTemplate( 'view' )->customFields( $cfields );
	}
	
	
	/**
	 * Get Price
	 *
	 * @return	\IPS\nexus\Money|NULL
	 */
	public function price()
	{
		$cost = $this->cost;
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->mbconf_nexus_on and $this->isPlaylist != 1 )
		{
			if ( $cost )
			{
				$currency = ( isset( $_SESSION['currency'] ) and \in_array( $_SESSION['currency'], \IPS\nexus\Money::currencies() ) ) ? $_SESSION['currency'] : \IPS\nexus\Customer::loggedIn()->defaultCurrency();
				if ( $costs = json_decode( $cost, TRUE ) )
				{
					if ( isset( $costs[ $currency ]['amount'] ) and $costs[ $currency ]['amount'] )
					{
						return new \IPS\nexus\Money( $costs[ $currency ]['amount'], $currency );
					}
				}
				else
				{
					return new \IPS\nexus\Money( $cost, $currency );
				}
			}
		}
		
		return NULL;
	}
	
	/**
	 * @brief	Number of purchases
	 */
	protected static $purchaseCounts;
	
	/**
	 * Get number of purchases
	 *
	 * @return	array
	 */
	public function purchaseCount()
	{
		if ( \IPS\Application::appIsEnabled( 'nexus' ) and \IPS\Settings::i()->mbconf_nexus_on and $this->isPlaylist != 1 )
		{
			if ( static::$purchaseCounts === NULL )
			{
				static::$purchaseCounts = iterator_to_array( \IPS\Db::i()->select( 'COUNT(*) AS count, ps_item_id', 'nexus_purchases', array( array( 'ps_app=? AND ps_type=?', 'musicbox', 'song' ), \IPS\Db::i()->in( 'ps_item_id', array_keys( static::$multitons ) ) ), NULL, NULL, 'ps_item_id' )->setKeyField('ps_item_id')->setValueField('count') );
				foreach ( array_keys( static::$multitons ) as $k )
				{
					if ( !isset( static::$purchaseCounts[ $k ] ) )
					{
						static::$purchaseCounts[ $k ] = 0;
					}
				}
			}
			
			if ( !isset( static::$purchaseCounts[ $this->id ] ) )
			{
				static::$purchaseCounts[ $this->id ] = \IPS\Db::i()->select( 'COUNT(*)', 'nexus_purchases', array( 'ps_app=? AND ps_type=? AND ps_item_id=?', 'musicbox', 'song', $this->id ) )->first();
			}

			return static::$purchaseCounts[ $this->id ];
		}
		
		return NULL;
	}
	
	public function playlistPurchaseCount()
	{
		$purchase = 0;
		
		$songs = $this->songsInPlaylist();
		
		if ( \count($songs) == 0  )
		{
			return $purchase;
		}
		
		$paidsongs = 0;
		$arr = array();
		
		foreach($songs as $s)
		{
			if ( $s->cost AND $s->price()->amount > 0)
			{
				$paidsongs++;
				if ( $s->purchaseCount() > 0 )
				{
					$arr[] = $s->purchaseCount();
				}
			}
		}

		if ( \count($arr) > 0 )
		{
			$purchase = min($arr);
		}

		return $purchase;
	}

	/**
	 * @brief	Can download?
	 */
	protected $canDownload = NULL;
	
	/**
	 * Can the member download this song?
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	bool
	 */
	public function canDownload( \IPS\Member $member = NULL )
	{
		if ( $this->canDownload === NULL )
		{
			try
			{
				$this->downloadCheck( $member );
				$this->canDownload = TRUE;
			}
			catch ( \DomainException $e )
			{
				$this->canDownload = FALSE;
			}
		}
		
		return $this->canDownload;
	}
	
	/**
	 * Download check
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	void
	 * @throws	\DomainException
	 */
	public function downloadCheck( \IPS\Member $member = NULL )
	{
		$member = $member ?: \IPS\Member::loggedIn();

		/* Basic permission check */
		if ( !$this->container()->can( 'download', $member ) )
		{
			throw new \DomainException( 'musicbox_error_download_noperm' );
		}

		/* Paid? */
		if ( \IPS\Settings::i()->mbconf_nexus_on and $this->isPlaylist != 1 and $this->cost and $this->price() and $member->member_id != $this->author()->member_id )
		{
			if ( !\count( \IPS\musicbox\extensions\nexus\Item\Song::getPurchases( \IPS\nexus\Customer::load( $member->member_id ), $this->id, FALSE ) ) )
			{
				throw new \DomainException( 'song_not_purchased' );
			}
		}
	}
	
	/**
	 * Can the member buy this song?
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	bool
	 */
	public function canBuy( \IPS\Member $member = NULL )
	{
		/* Is this a paid file? */
		if ( !\IPS\Application::appIsEnabled( 'nexus' ) or !\IPS\Settings::i()->mbconf_nexus_on or $this->isPlaylist == 1 or !$this->cost or !$this->price() )
		{
			return FALSE;
		}
		
		/* Init */
		$member = $member ?: \IPS\Member::loggedIn();

        /* File author */
        if( $member == $this->author() )
        {
            return FALSE;
        }
		
		/* Basic permission check */
		if ( !$this->container()->can( 'download', $member ) )
		{
			/* Hold on - if we're a guest and buying means we'll have to register which will put us in a group with permission, we can continue */
			if ( \IPS\Member::loggedIn()->member_id )
			{
				return FALSE;
			}
		}

		return TRUE;
	}
	
	/**
	 * Can the member buy this playlist?
	 *
	 * @param	\IPS\Member|NULL	$member		The member to check or NULL for currently logged in member
	 * @return	bool
	 */
	public function canBuyPlaylist( \IPS\Member $member = NULL )
	{
		/* Is this a playlis? */
		if ( !\IPS\Application::appIsEnabled( 'nexus' ) or !\IPS\Settings::i()->mbconf_nexus_on or $this->isPlaylist != 1 )
		{
			return FALSE;
		}
		
		/* Are there any paid songs in the playlist ? */
		$songs = $this->songsInPlaylist();
		
		if ( \count($songs) == 0  )
		{
			return FALSE;
		}
		
		$paidsongs = 0;
		
		foreach($songs as $s)
		{
			if ( $s->cost AND $s->price()->amount > 0 AND !$s->canDownload())
			{
				$paidsongs++;
			}
		}

		if ( $paidsongs == 0 )
		{
			return FALSE;
		}
		
		/* Init */
		$member = $member ?: \IPS\Member::loggedIn();

        /* File author */
        if( $member == $this->author() )
        {
            return FALSE;
        }
		
		/* Basic permission check */
		if ( !$this->container()->can( 'download', $member ) )
		{
			/* Hold on - if we're a guest and buying means we'll have to register which will put us in a group with permission, we can continue */
			if ( \IPS\Member::loggedIn()->member_id or !$this->container()->can( 'download', \IPS\Member\Group::load( \IPS\Settings::i()->member_group ) ) )
			{
				return FALSE;
			}
		}

		return TRUE;
	}
		
	/* !Tags */
	
	/**
	 * Can tag?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	bool
	 */
	public static function canTag( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canTag( $member, $container ) and ( $container === NULL or !$container->tags_disabled );
	}
	
	/**
	 * Can use prefixes?
	 *
	 * @param	\IPS\Member|NULL		$member		The member to check for (NULL for currently logged in member)
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	bool
	 */
	public static function canPrefix( \IPS\Member $member = NULL, \IPS\Node\Model $container = NULL )
	{
		return parent::canPrefix( $member, $container ) and ( $container === NULL or !$container->tags_noprefixes );
	}
	
	/**
	 * Defined Tags
	 *
	 * @param	\IPS\Node\Model|NULL	$container	The container to check if tags can be used in, if applicable
	 * @return	array
	 */
	public static function definedTags( \IPS\Node\Model $container = NULL )
	{
		if ( $container and $container->tags_predefined )
		{
			return explode( ',', $container->tags_predefined );
		}
		
		return parent::definedTags( $container );
	}
		
	/**
	 * Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		if ( $this->file )
		{
			try
			{			
				\IPS\File::get( 'musicbox_File', $this->file )->delete();
			}
			catch( \Exception $e ){}
		}
		if ( $this->image )
		{
			try
			{			
				\IPS\File::get( 'musicbox_Playlist', $this->playlist_image )->delete();
				\IPS\File::get( 'musicbox_Playlist', $this->playlist_image_thumb )->delete();
			}
			catch( \Exception $e ){}
		}
		if ( $this->isPlaylist == 1 && \count($this->songsInPlaylist()) > 0) // Xoa playlist thi xoa luon ID playlist cua bai hat co trong list
		{
			$pl = array();
			$updatePL = array();
			foreach( $this->songsInPlaylist() as $s )
			{
				$pl = explode(",", $s->inPlaylist);
				$updatePL = array_diff($pl, array($this->id));
				$s->inPlaylist = \count($updatePL) > 0 ? implode(",", $updatePL) : null;
				$s->save();				
			}
		}
		parent::delete();
	}

	/**
	 * Move
	 *
	 * @param	\IPS\Node\Model	$container	Container to move to
	 * @param	bool			$keepLink	If TRUE, will keep a link in the source
	 * @return	void
	 */
	public function move( \IPS\Node\Model $container, $keepLink=FALSE )
	{
		if(	$container->bitoptions['containerMode'] == 1 )
		{
			throw new \InvalidArgumentException;
		}
		
		parent::move( $container, $keepLink );
	}
	
	/*-------------------------------------------------------------------------*/
	// get thumbnail
	/*-------------------------------------------------------------------------*/
	public function thumbnail($object=0)
	{
		if ( $this->isPlaylist == 1 && $this->playlist_image_thumb )
		{
			return $object == 1 ? \IPS\File::get( 'musicbox_Playlist', $this->playlist_image_thumb ) : (string) \IPS\File::get( 'musicbox_Playlist', $this->playlist_image_thumb )->url;
		}
		
		if ( $this->image )
		{
			$file = $this->image_thumb ? \IPS\File::get( 'musicbox_Image', $this->image_thumb ) : \IPS\File::get( 'musicbox_Image', $this->image );
			
			if ( $object == 1 )
			{
				return $file;
			}
			
			return (string) $file->url;
		}
		
		if ( $this->singer )
		{
			foreach( explode(",", trim($this->singer) . ",") as $name )
			{
				try
				{
					$artist = \IPS\Db::i()->select( '*', 'musicbox_artists', array( 'artist_name=?', $name ) )->first();
				}
				catch ( \UnderflowException $e ){}
				if ( $artist['artist_avatar_thumb'] )
				{
					$thumbnail = $object == 1 ? \IPS\File::get( 'musicbox_Artist', $artist['artist_avatar_thumb'] ) : (string) \IPS\File::get( 'musicbox_Artist', $artist['artist_avatar_thumb'] )->url;
					break;
				}
			}
		}
		
		if ( $thumbnail )
		{
			return $thumbnail;
		}
		else
		{
			return $object == 1 ? null : (string) \IPS\Theme::i()->resource( 'no-cover.png', 'musicbox' );
		}
	}	

	public function image()
	{
		if ( $this->image )
		{
			return (string) \IPS\File::get( 'musicbox_Image', $this->image )->url;
		}
		
		if ( $this->singer )
		{
			foreach( explode(",", trim($this->singer) . ",") as $name )
			{
				try
				{
					$artist = \IPS\Db::i()->select( '*', 'musicbox_artists', array( 'artist_name=?', $name ) )->first();
				}
				catch ( \UnderflowException $e ){}
				
				if ( $artist['artist_avatar'] )
				{
					$thumbnail = (string) \IPS\File::get( 'musicbox_Artist', $artist['artist_avatar'] )->url;
					break;
				}
			}
		}
		
		return $thumbnail ? $thumbnail : (string) \IPS\Theme::i()->resource( 'no-cover.png', 'musicbox' );
	}	
	
	/*-------------------------------------------------------------------------*/
	// get file
	/*-------------------------------------------------------------------------*/
	public function musicFile()
	{
		$playfile = $this->file;
		$isExternal = $this->external;
		
		if ( \IPS\Settings::i()->mbconf_nexus_on and $this->isPlaylist != 1 and $this->cost and $this->price() and \IPS\Member::loggedIn()->member_id != $this->author()->member_id )
		{
			if ( !\count( \IPS\musicbox\extensions\nexus\Item\Song::getPurchases( \IPS\nexus\Customer::load( \IPS\Member::loggedIn()->member_id ), $this->id, FALSE ) ) )
			{
				$playfile = $this->demo;
				$isExternal = 0;
			}
		}
		
		if ( $isExternal == 1  )
		{
			if ( $playfile && !$this->filesize )
			{
				$this->filesize = \IPS\Application::load('musicbox')->remotefileSize($playfile);
				$this->save();
			}
			return $playfile;
		}
		else
		{	
			$file = \IPS\File::get( 'musicbox_File', $playfile );
			if ( !$this->filesize )
			{
				$this->filesize = $file->filesize();
				$this->save();
			}	

			//return $this->url('play')->csrf()->setQueryString( array('name' => "play." . $this->fileExt() ) );
			return (string) $file->url;
		}
	}		
	
	/*-------------------------------------------------------------------------*/
	// get file extension
	/*-------------------------------------------------------------------------*/
	public function fileExt()
	{
		$supportExt = array('mp3', 'wma', 'm4a');
		$file = mb_strtolower($this->file);
		$end = mb_substr(mb_strrchr($this->file,'.'),1);
		if ( \in_array( $end, $supportExt ) )
		{
			return $end;
		}
		else
		{
			foreach( $supportExt as $e )
			{
				if ( mb_strpos( $file, '.' . $e ) !== false || mb_strpos( $file, '_' . $e ) !== false )
				{
					return $e;
				}
			}			
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Singers
	/*-------------------------------------------------------------------------*/
	public function singers()
	{
		return $this->singer ? $this->singer : \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_unknown' );
	}
	
	/*-------------------------------------------------------------------------*/
	// Artists
	/*-------------------------------------------------------------------------*/
	public function artistLinks( $txt )
	{
		if ( !$txt )
		{
			return \IPS\Member::loggedIn()->language()->addToStack( 'musicbox_unknown' );
		}
		
		foreach ( explode(",", $txt . ",") as $name )
		{
			if ( $name )
			{
				$url = \IPS\Http\Url::internal( "app=musicbox&module=musicbox&controller=artists&name=" . trim($name), 'front', 'musicbox_artists' );
				$return[] = "<a href='" . $url . "' rel='loadAjaxPage'>" . $name . "</a>";
			}
		}
		
		$artists = implode(", ", $return);
		return $artists;
	}
	
	/*-------------------------------------------------------------------------*/
	// Playlist info
	/*-------------------------------------------------------------------------*/
	public function playlistInfo()
	{
		$playlist = array();
		if ( $this->inPlaylist )
		{
			foreach( explode(",", $this->inPlaylist) as $pl )
			{
				try
				{
					$playlist[] = \IPS\musicbox\Song::load( $pl );
				}
				catch ( \OutOfRangeException $e ){}
			}
		}
		return $playlist;
	}
	
	/*-------------------------------------------------------------------------*/
	// songs in playlist
	/*-------------------------------------------------------------------------*/
	public function songsInPlaylist()
	{
		$data = array();
		if ( $this->isPlaylist == 1 )
		{
			foreach( \IPS\Db::i()->select( '*', 'musicbox_songs', "find_in_set('{$this->id}',song_inPlaylist) <> 0", 'song_playlist_pos ASC')  as $s )
			{
				$data[] = \IPS\musicbox\Song::constructFromData( $s );
			}
		}

		return $data;
	}
	
	/*-------------------------------------------------------------------------*/
	// Check this playlist in my favorite
	/*-------------------------------------------------------------------------*/	
	public function playlistInFavorite()
	{
		try
		{
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', array( 'mpl_mem=? and mpl_playlistID=?', \IPS\Member::loggedIn()->member_id, $this->id ) )->first();
		}
		catch ( \UnderflowException $e ){}
		if ( $myplaylist['mpl_id'] > 0 )
		{
			return $myplaylist;
		}
		else return null;
	}
	
	/*-------------------------------------------------------------------------*/
	// truncateText
	/*-------------------------------------------------------------------------*/
	public function truncateText( $text, $chars=300 )
	{	
		$text = $text." ";
		$text = mb_substr($text, 0, $chars);
		$text = mb_substr($text, 0, mb_strrpos($text,' '));
		$text = $text."...";
		return $text;
	}
	
	/*-------------------------------------------------------------------------*/
	// My Favorite songs
	/*-------------------------------------------------------------------------*/	
	public static function myFavoriteSongs($playlistID, $owner=NULL)
	{
		try
		{
			$playlistID = $playlistID ? (int) $playlistID : 0;
			$where[] = array( 'mpl_id=?', $playlistID );
			if ( $owner )
			{
				$where[] = array( 'mpl_mem=?', $owner );
			}
			
			$myplaylist = \IPS\Db::i()->select( '*', 'musicbox_favorite', $where )->first();
			
			if ( $myplaylist['mpl_songs'] )
			{
				foreach( explode(',', $myplaylist['mpl_songs']) as $id )
				{
					try
					{
						$song = \IPS\musicbox\Song::load( $id );
						$data[] = $song;
					}
					catch ( \OutOfRangeException $e ){
						$pl = explode(",", $myplaylist['mpl_songs']);
						$allSongs = array_diff($pl, array($song->id));					
						\IPS\Db::i()->update( 'musicbox_favorite', array('mpl_songs' => implode(",", $allSongs)), array( 'mpl_id=?', $myplaylist['mpl_id'] ) );
						$broken++;
					}
				}
				if( $broken > 0 )
				{
					\IPS\Output::i()->redirect( isset( $_SERVER['HTTP_REFERER'] ) ? $_SERVER['HTTP_REFERER'] : \IPS\Http\Url::internal( '' ) );
				}
				return $data;
			}
		}
		catch ( \UnderflowException $e ){}
		
		return NULL;
	}
	
	/**
	 * Give a content item the opportunity to filter similar content
	 * 
	 * @note Intentionally blank but can be overridden by child classes
	 * @return array|NULL
	 */
	public function similarContentFilter()
	{
		return array(
			array( 'tag_meta_app=? AND tag_meta_area=? AND tag_meta_id=?', 'musicbox', 'songs', $this->id )
		);
	}	

	
}