<?php
/**
 * @brief		mbrssimport Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	musicbox
 * @since		31 Dec 2016
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\musicbox\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * mbrssimport Task
 */
class _mbrssimport extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		$timeCheck = new \IPS\DateTime;
		$timeCheck->sub( new \DateInterval( 'PT10M' ) );

		$this->runUntilTimeout(function() use ( $timeCheck ) {
			try
			{
				$feed = \IPS\musicbox\Feed::constructFromData( \IPS\Db::i()->select( '*', 'musicbox_rss_import', array( 'mbrss_import_enabled=1 AND mbrss_import_last_import<?', $timeCheck->getTimestamp() ), 'mbrss_import_last_import ASC', 1 )->first() );
				$feed->run();
			}
			/* There's nothing more left to process */
			catch ( \UnderflowException $e )
			{
				/* Check to see if there are any feeds.. If no feed, we can disable the task */
				if( !\IPS\Db::i()->select( 'count(mbrss_import_id)', 'musicbox_rss_import', array( 'mbrss_import_enabled=1' ), NULL, 1 )->first() )
				{
					\IPS\Db::i()->update( 'core_tasks', array( 'enabled' => 0 ), array( '`key`=?', 'mbrssimport' ) );
				}

				/* No further processing needed */
				return FALSE;
			}
			/* Any other exception means an error which should be logged */
			catch ( \Exception $e )
			{
				/* If there is an error, we need to log it but the error should not prevent other feeds from importing */
				if ( isset( $feed ) AND ( $feed instanceof \IPS\musicbox\Feed ) )
				{
					$feed->last_import = time();
					$feed->save();
				}
				throw new \IPS\Task\Exception( $this, $e->getMessage() );
			}

			/* Run again to see if there's anything left */
			return TRUE;
		});
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}