<?php
/**
 * @brief		musicboxFeed Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	musicbox
 * @since		30 Apr 2019
 */

namespace IPS\musicbox\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * musicboxFeed Widget
 */
class _musicboxFeed extends \IPS\Widget\PermissionCache
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'musicboxFeed';
	
	/**
	 * @brief	App
	 */
	public $app = 'musicbox';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		$form = parent::configuration( $form );

		$form->add( new \IPS\Helpers\Form\Text( 'mbox_block_title', $this->configuration['mbox_block_title'], TRUE ) );
		
		$form->add( new \IPS\Helpers\Form\Number( 'mbox_block_limit', $this->configuration['mbox_block_limit'] ? $this->configuration['mbox_block_limit'] : 10, TRUE ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'mbox_block_type', $this->configuration['mbox_block_type'], TRUE, array(
			'options' => array(
				'songs'      	 => 'mbox_block_type_songs',
			    'playlists'      => 'mbox_block_type_playlists'
			)
		) ) );	
		
		if( \IPS\Settings::i()->tags_enabled )
		{
			if ( \IPS\Settings::i()->tags_force_lower )
			{
				$options['autocomplete']['forceLower'] = TRUE;
			}

			if ( \IPS\Settings::i()->tags_clean )
			{
				$options['autocomplete']['filterProfanity'] = TRUE;
			}

			$options['autocomplete']['prefix'] = FALSE;
			$options['autocomplete']['minimized'] = FALSE;
			
			$form->add( new \IPS\Helpers\Form\Text( 'widget_feed_tags', ( isset( $this->configuration['widget_feed_tags'] ) ? $this->configuration['widget_feed_tags'] : array( 'tags' => NULL ) ), FALSE, $options ) );
		}
		
		$form->add( new \IPS\Helpers\Form\Node( 'mbox_block_cats', isset( $this->configuration['mbox_block_cats'] ) ? $this->configuration['mbox_block_cats'] : 0, FALSE, array(
			'class'           => '\IPS\musicbox\Category',
			'zeroVal'         => 'all',
			'permissionCheck' => 'view',
			'multiple'        => true,
			'forceOwner'	  => false,
			'clubs'			  => true,
		) ) );
			
		$form->add( new \IPS\Helpers\Form\Select( 'mbox_sort_on', isset( $this->configuration['mbox_sort_on'] ) ? $this->configuration['mbox_sort_on'] : 'v_updated', TRUE, array(
            'options' => array(
	            'song_started'     		=> 'mbox_sort_on_started',
	            'song_title'    		=> 'mbox_sort_on_title',
	            'song_rating_average'	=> 'mbox_sort_on_rating',
	            'song_comments'  		=> 'mbox_sort_on_comments',
				'song_last_comment'		=> 'mbox_sort_on_last_comment',
	            'song_views'       		=> 'mbox_sort_on_views'
            )
        ) ) );

		$form->add( new \IPS\Helpers\Form\Select( 'mbox_sort_dir', isset( $this->configuration['mbox_sort_dir'] ) ? $this->configuration['mbox_sort_dir'] : 'desc', TRUE, array(
            'options' => array(
	            'desc'   => 'mbox_sort_dir_dsc',
	            'asc'    => 'mbox_sort_dir_asc'
            )
        ) ) );
		
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		if ( \is_array( $values['mbox_block_cats'] ) )
 		{
	 		$values['mbox_block_cats'] = array_keys( $values['mbox_block_cats'] );
 		}
		
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		if ( $this->configuration['mbox_block_type'] == 'playlists' )
		{
			$where[] = array( 'song_isPlaylist=?', 1 );
		}
		else
		{
			$where[] = array( 'song_isPlaylist=?', 0 );
		}
		
		if ( $this->configuration['mbox_block_cats'] != 0 )
		{
			$where[] = array( 'song_cat IN(' . implode(',', $this->configuration['mbox_block_cats']) . ')' ) ;
		}
		
		if ( \IPS\Settings::i()->tags_enabled and isset( $this->configuration['widget_feed_tags'] ) and \is_array( $this->configuration['widget_feed_tags'] ) and \count( $this->configuration['widget_feed_tags'] ) )
		{
			$tagWhere = array();
			$binds    = array( 'musicbox', 'musicbox' );
			foreach( $this->configuration['widget_feed_tags'] as $tag )
			{
				$tagWhere[] = "( tag_text LIKE CONCAT( ?, '%') )";
				$binds[] = $tag;
			}
			
			$where[] = array( 'musicbox_songs.song_id IN (?)', \IPS\Db::i()->select( 'tag_meta_id', 'core_tags', array( array_merge( array( 'tag_meta_app=? AND tag_meta_area=? AND (' . implode( ' OR ', $tagWhere ) . ')' ), $binds ) ) ) );
		}
		
		if ( $this->configuration['mbox_sort_on'] && $this->configuration['mbox_sort_dir'] )
		{
			$order = $this->configuration['mbox_sort_on'] . ' ' . $this->configuration['mbox_sort_dir'];
		}
		
		$data = \IPS\musicbox\Song::getItemsWithPermission( $where, $order, $this->configuration['mbox_block_limit'], 'read', \IPS\Content\Hideable::FILTER_AUTOMATIC, 0, NULL, TRUE );

		if ( \count( $data ) )
		{		
			return $this->output( $data, $this->configuration );
		}
		else
		{
			return '';
		}
	}
	
	
	
	
	
	
}